@@ -1,53 +1,38 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-import time
-import datetime
 import tryton.rpc as rpc
-from tryton.wizard import Wizard
-from tryton.common import message, error, selection, file_open, mailto
+from tryton.common import message, selection, file_open, mailto
 from tryton.gui.window import Window
 from tryton.pyson import PYSONDecoder
 import gettext
 import tempfile
-import base64
 import os
 import webbrowser
-import tryton.common as common
+from tryton.pyson import PYSONEncoder
+from tryton.common import RPCProgress, RPCExecute, RPCException
 
 _ = gettext.gettext
 
+
 class Action(object):
 
     @staticmethod
-    def exec_report(name, data, window, direct_print=False, email_print=False,
+    def exec_report(name, data, direct_print=False, email_print=False,
             email=None, context=None):
         if context is None:
             context = {}
         if email is None:
             email = {}
         data = data.copy()
-        ids = data['ids']
-        del data['ids']
         ctx = rpc.CONTEXT.copy()
         ctx.update(context)
-        if not ids:
-            args = ('model', data['model'], 'search', [], 0, None, None, ctx)
-            try:
-                ids = rpc.execute(*args)
-            except Exception, exception:
-                ids = common.process_exception(exception, window, *args)
-                if not ids:
-                    return False
-            if ids == []:
-                message(_('Nothing to print!'), window)
-                return False
-            data['id'] = ids[0]
-        args = ('report', name, 'execute', ids, data, ctx)
-        rpcprogress = common.RPCProgress('execute', args, window)
+        ctx['direct_print'] = direct_print
+        ctx['email_print'] = email_print
+        ctx['email'] = email
+        args = ('report', name, 'execute', data.get('ids', []), data, ctx)
         try:
-            res = rpcprogress.run()
-        except Exception, exception:
-            common.process_exception(exception, window)
+            res = RPCProgress('execute', args).run()
+        except RPCException:
             return False
         if not res:
             return False
@@ -58,44 +43,38 @@ class Action(object):
         fp_name = os.path.join(dtemp,
                 name.replace(os.sep, '_').replace(os.altsep or os.sep, '_') \
                         + os.extsep + type)
-        file_d = open(fp_name, 'wb')
-        file_d.write(base64.decodestring(data))
-        file_d.close()
+        with open(fp_name, 'wb') as file_d:
+            file_d.write(data)
         if email_print:
             mailto(to=email.get('to'), cc=email.get('cc'),
                     subject=email.get('subject'), body=email.get('body'),
                     attachment=fp_name)
         else:
-            file_open(fp_name, type, window, print_p=print_p)
+            file_open(fp_name, type, print_p=print_p)
         return True
 
     @staticmethod
-    def execute(act_id, data, window, action_type=None, context=None):
-        if context is None:
-            context = {}
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(context)
+    def execute(act_id, data, action_type=None, context=None):
         if not action_type:
             res = False
             try:
-                res = rpc.execute('model', 'ir.action', 'read', act_id,
-                        ['type'], ctx)
-            except Exception, exception:
-                common.process_exception(exception, window)
+                res = RPCExecute('model', 'ir.action', 'read', act_id,
+                    ['type'], context=context)
+            except RPCException:
                 return
             if not res:
-                raise Exception, 'ActionNotFound'
+                raise Exception('ActionNotFound')
             action_type = res['type']
         try:
-            res = rpc.execute('model', action_type, 'search_read',
-                    [('action', '=', act_id)], 0, 1, None, None, ctx)
-        except Exception, exception:
-            common.process_exception(exception, window)
+            res = RPCExecute('model', action_type, 'search_read',
+                [('action', '=', act_id)], 0, 1, None, None,
+                context=context)
+        except RPCException:
             return
-        Action._exec_action(res, window, data)
+        Action._exec_action(res, data)
 
     @staticmethod
-    def _exec_action(action, window, data=None, context=None):
+    def _exec_action(action, data=None, context=None):
         if context is None:
             context = {}
         if data is None:
@@ -121,6 +100,7 @@ class Action(object):
             }
             ctx.update(rpc.CONTEXT)
             eval_ctx = ctx.copy()
+            eval_ctx['_user'] = rpc._USER
             action_ctx = PYSONDecoder(eval_ctx).decode(
                     action.get('pyson_context') or '{}')
             ctx.update(action_ctx)
@@ -128,12 +108,15 @@ class Action(object):
 
             domain_context = ctx.copy()
             domain_context['context'] = ctx
-            domain = PYSONDecoder(domain_context).decode(action['pyson_domain'])
+            domain_context['_user'] = rpc._USER
+            domain = PYSONDecoder(domain_context).decode(
+                action['pyson_domain'])
 
             search_context = ctx.copy()
             search_context['context'] = ctx
+            search_context['_user'] = rpc._USER
             search_value = PYSONDecoder(search_context).decode(
-                    action['pyson_search_value'] or '{}')
+                    action['pyson_search_value'] or '[]')
 
             name = False
             if action.get('window_name', True):
@@ -142,30 +125,22 @@ class Action(object):
             res_model = action.get('res_model', data.get('res_model'))
             res_id = action.get('res_id', data.get('res_id'))
 
-            Window.create(view_ids, res_model, res_id, domain, window,
+            Window.create(view_ids, res_model, res_id, domain,
                     action_ctx, view_mode, name=name,
                     limit=action.get('limit'),
                     auto_refresh=action.get('auto_refresh'),
                     search_value=search_value,
                     icon=(action.get('icon.rec_name') or ''))
         elif action['type'] == 'ir.action.wizard':
-            if action.get('window', False):
-                Window.create_wizard(action['wiz_name'], data, window,
-                    direct_print=action.get('direct_print', False),
-                    email_print=action.get('email_print', False),
-                    email=action.get('email'), name=action.get('name', False),
-                    context=context, icon=(action.get('icon.rec_name') or ''))
-            else:
-                Wizard.execute(action['wiz_name'], data, window,
-                        direct_print=action.get('direct_print', False),
-                        email_print=action.get('email_print', False),
-                        email=action.get('email'), context=context)
-            if action['wiz_name'] == 'ir.ui.view_sc.add':
-                from tryton.gui.main import Main
-                Main.get_main().shortcut_set()
+            Window.create_wizard(action['wiz_name'], data,
+                direct_print=action.get('direct_print', False),
+                email_print=action.get('email_print', False),
+                email=action.get('email'), name=action.get('name', False),
+                context=context, icon=(action.get('icon.rec_name') or ''),
+                window=action.get('window', False))
 
         elif action['type'] == 'ir.action.report':
-            Action.exec_report(action['report_name'], data, window,
+            Action.exec_report(action['report_name'], data,
                     direct_print=action.get('direct_print', False),
                     email_print=action.get('email_print', False),
                     email=action.get('email'), context=context)
@@ -175,29 +150,56 @@ class Action(object):
                 webbrowser.open(action['url'], new=2)
 
     @staticmethod
-    def exec_keyword(keyword, window, data=None, context=None, warning=True,
+    def exec_keyword(keyword, data=None, context=None, warning=True,
             alwaysask=False):
         actions = []
         if 'id' in data:
             model_id = data.get('id', False)
             try:
-                actions = rpc.execute('model', 'ir.action.keyword',
-                        'get_keyword', keyword, (data['model'], model_id),
-                        rpc.CONTEXT)
-            except Exception, exception:
-                common.process_exception(exception, window)
+                actions = RPCExecute('model', 'ir.action.keyword',
+                    'get_keyword', keyword, (data['model'], model_id))
+            except RPCException:
                 return False
 
         keyact = {}
         for action in actions:
             keyact[action['name'].replace('_', '')] = action
 
-        res = selection(_('Select your action'), keyact, window,
-                alwaysask=alwaysask)
+        res = selection(_('Select your action'), keyact, alwaysask=alwaysask)
         if res:
             (name, action) = res
-            Action._exec_action(action, window, data, context=context)
+            Action._exec_action(action, data, context=context)
             return (name, action)
         elif not len(keyact) and warning:
-            message(_('No action defined!'), window)
+            message(_('No action defined!'))
         return False
+
+    @staticmethod
+    def evaluate(action, atype, record):
+        '''
+        Evaluate the action with the record.
+        '''
+        action = action.copy()
+        if atype in ('print', 'action'):
+            email = {}
+            if 'pyson_email' in action:
+                email = record.expr_eval(action['pyson_email'])
+                if not email:
+                    email = {}
+            if 'subject' not in email:
+                email['subject'] = action['name'].replace('_', '')
+            action['email'] = email
+        elif atype == 'relate':
+            encoder = PYSONEncoder()
+            if 'pyson_domain' in action:
+                action['pyson_domain'] = encoder.encode(
+                    record.expr_eval(action['pyson_domain'], check_load=False))
+            if 'pyson_context' in action:
+                action['pyson_context'] = encoder.encode(
+                    record.expr_eval(action['pyson_context'],
+                        check_load=False))
+
+        else:
+            raise NotImplementedError("Action type '%s' is not supported" %
+                atype)
+        return action
