@@ -11,13 +11,17 @@ import sys
 import locale
 import gtk
 
+from tryton.exceptions import TrytonError
+
 _ = gettext.gettext
 
+
 def get_home_dir():
     if os.name == 'nt':
         return os.path.join(os.environ['HOMEDRIVE'], os.environ['HOMEPATH'])
     return os.environ['HOME']
 
+
 def get_config_dir():
     if os.name == 'nt':
         return os.path.join(os.environ['APPDATA'], '.config', 'tryton',
@@ -27,21 +31,6 @@ def get_config_dir():
 if not os.path.isdir(get_config_dir()):
     os.makedirs(get_config_dir(), 0700)
 
-def find_path(progs, args):
-    if os.name == 'nt':
-        return ''
-    if os.name == 'mac' or \
-            (hasattr(os, 'uname') and os.uname()[0] == 'Darwin'):
-        return ''
-    paths = [x for x in os.environ['PATH'].split(':')
-            if os.path.isdir(x)]
-    for dir in paths:
-        for prog in progs:
-            val = os.path.join(dir, prog)
-            if os.path.isfile(val) or os.path.islink(val):
-                return val + ' ' + args
-    return ''
-
 
 class ConfigManager(object):
     "Config manager"
@@ -50,11 +39,12 @@ class ConfigManager(object):
         short_version = '.'.join(VERSION.split('.', 2)[:2])
         demo_server = 'demo%s.tryton.org' % short_version
         demo_database = 'demo%s' % short_version
+        form_tab = 'left' if os.name != 'nt' else 'top'
         self.defaults = {
             'login.profile': demo_server,
             'login.login': 'demo',
             'login.server': demo_server,
-            'login.port': '8070',
+            'login.port': '8000',
             'login.db': demo_database,
             'login.expanded': False,
             'tip.autostart': False,
@@ -68,24 +58,14 @@ class ConfigManager(object):
             'client.default_height': 750,
             'client.modepda': False,
             'client.toolbar': 'default',
-            'client.form_tab': 'left',
+            'client.form_tab': form_tab,
             'client.maximize': False,
             'client.save_width_height': True,
+            'client.save_tree_expanded_state': True,
             'client.spellcheck': False,
             'client.default_path': get_home_dir(),
             'client.lang': locale.getdefaultlocale()[0],
             'client.language_direction': 'ltr',
-            'client.actions': {
-                'odt': {0: find_path(['ooffice', 'ooffice2', 'libreoffice'], '"%s"'),
-                    1: find_path(['ooffice', 'ooffice2', 'libreoffice'], '-p "%s"')},
-                'txt': {0: find_path(['ooffice', 'ooffice2', 'libreoffice'], '"%s"'),
-                    1: find_path(['ooffice', 'ooffice2', 'libreoffice'], '-p "%s"')},
-                'pdf': {0: find_path(['evince', 'xpdf', 'gpdf',
-                    'kpdf', 'epdfview', 'acroread'], '"%s"'), 1: ''},
-                'png': {0: find_path(['feh', 'display', 'qiv', 'eye'], '"%s"'), 1: ''},
-                'csv': {0: find_path(['ooffice', 'ooffice2', 'libreoffice'], '"%s"'),
-                    1: find_path(['ooffice', 'ooffice2', 'libreoffice'], '-p "%s"')},
-                },
             'client.email': '',
             'client.can_change_accelerators': False,
             'client.limit': 1000,
@@ -121,22 +101,23 @@ class ConfigManager(object):
         opt, self.arguments = parser.parse_args()
 
         if len(self.arguments) > 1:
-            raise Exception(_('Too much arguments'))
+            raise TrytonError(_('Too much arguments'))
 
         if opt.config and not os.path.isfile(opt.config):
-            raise Exception(_('File "%s" not found') % (opt.config,))
-        self.rcfile = opt.config or os.path.join(get_config_dir(), 'tryton.conf')
+            raise TrytonError(_('File "%s" not found') % (opt.config,))
+        self.rcfile = opt.config or os.path.join(
+            get_config_dir(), 'tryton.conf')
         self.load()
 
         for arg in ('log_level', 'log_logger'):
             if getattr(opt, arg):
-                self.options['logging.'+arg[4:]] = getattr(opt, arg)
+                self.options['logging.' + arg[4:]] = getattr(opt, arg)
         if opt.verbose:
             self.options['logging.default'] = 'INFO'
 
         for arg in ('login', 'port', 'server'):
             if getattr(opt, arg):
-                self.options['login.'+arg] = getattr(opt, arg)
+                self.options['login.' + arg] = getattr(opt, arg)
 
     def save(self):
         try:
@@ -149,7 +130,7 @@ class ConfigManager(object):
                     configparser.add_section(section)
                 configparser.set(section, name, self.config[entry])
             configparser.write(open(self.rcfile, 'wb'))
-        except Exception:
+        except IOError:
             logging.getLogger('common.options').warn(
                     _('Unable to write config file %s!') % \
                             (self.rcfile,))
@@ -180,7 +161,8 @@ class ConfigManager(object):
             self.defaults.get(key)))
 
 CONFIG = ConfigManager()
-if os.name == 'nt' and hasattr(sys, 'frozen'):
+if (os.name == 'nt' and hasattr(sys, 'frozen')
+        and os.path.basename(sys.executable) == 'tryton.exe'):
     CURRENT_DIR = os.path.dirname(unicode(sys.executable,
         sys.getfilesystemencoding()))
 else:
@@ -200,6 +182,7 @@ if not os.path.isdir(PIXMAPS_DIR):
 TRYTON_ICON = gtk.gdk.pixbuf_new_from_file(
         os.path.join(PIXMAPS_DIR, 'tryton-icon.png').encode('utf-8'))
 
+
 def _data_dir():
     data_dir = os.path.join(CURRENT_DIR, 'share', 'tryton')
     if not os.path.isdir(data_dir):
