@@ -5,10 +5,7 @@ import ConfigParser
 import gtk
 import gobject
 import os
-import re
 import gettext
-import threading
-import time
 
 from tryton.version import VERSION
 import tryton.common as common
@@ -26,13 +23,15 @@ class DBListEditor(object):
         self.profiles = profiles
         self.current_database = None
         self.old_profile, self.current_profile = None, None
+        self.db_cache = None
         self.updating_db = False
 
         # GTK Stuffs
         self.parent = parent
         self.dialog = gtk.Dialog(title=_(u'Profile Editor'), parent=parent,
-            flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT)
-        self.ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT)
+            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+        self.ok_button = self.dialog.add_button(gtk.STOCK_OK,
+            gtk.RESPONSE_ACCEPT)
         self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.dialog.set_has_separator(True)
         self.dialog.set_icon(TRYTON_ICON)
@@ -41,9 +40,7 @@ class DBListEditor(object):
         vbox_profiles = gtk.VBox(homogeneous=False, spacing=6)
         self.cell = gtk.CellRendererText()
         self.cell.set_property('editable', True)
-        self.cell.connect('edited', self.edit_profilename)
         self.cell.connect('editing-started', self.edit_started)
-        self.cell.connect('editing-canceled', self.edit_canceled)
         self.profile_tree = gtk.TreeView()
         self.profile_tree.set_model(profile_store)
         self.profile_tree.insert_column_with_attributes(-1, _(u'Profile'),
@@ -94,7 +91,8 @@ class DBListEditor(object):
         database.set_padding(3, 3)
         self.database_entry = gtk.Entry()
         self.database_entry.connect('changed', self.dbentry_changed)
-        self.database_entry.connect('changed', self.update_profiles, 'database')
+        self.database_entry.connect('changed', self.update_profiles,
+            'database')
         self.database_entry.set_activates_default(True)
         self.database_label = gtk.Label()
         self.database_label.set_use_markup(True)
@@ -108,7 +106,7 @@ class DBListEditor(object):
         self.database_combo.connect('changed', self.dbcombo_changed)
         self.database_button = gtk.Button(_(u'Create'))
         self.database_button.connect('clicked', self.db_create)
-        self.database_progressbar= gtk.ProgressBar()
+        self.database_progressbar = gtk.ProgressBar()
         self.database_progressbar.set_text(_(u'Fetching databases list'))
         image = gtk.Image()
         image.set_from_stock('tryton-new', gtk.ICON_SIZE_BUTTON)
@@ -131,7 +129,8 @@ class DBListEditor(object):
         username.set_alignment(1, 0.5)
         username.set_padding(3, 3)
         self.username_entry = gtk.Entry()
-        self.username_entry.connect('changed', self.update_profiles, 'username')
+        self.username_entry.connect('changed', self.update_profiles,
+            'username')
         self.username_entry.set_activates_default(True)
         table.attach(username, 0, 1, 4, 5, yoptions=False, xoptions=gtk.FILL)
         table.attach(self.username_entry, 1, 2, 4, 5, yoptions=False)
@@ -174,10 +173,12 @@ class DBListEditor(object):
     def clear_entries(self):
         for entryname in ('host', 'port', 'database', 'username'):
             entry = getattr(self, '%s_entry' % entryname)
+            entry.handler_block_by_func(self.update_profiles)
             if entryname == 'port':
                 entry.set_text('8000')
             else:
                 entry.set_text('')
+            entry.handler_unblock_by_func(self.update_profiles)
         self.current_database = None
         self.database_combo.set_active(-1)
         self.database_combo.get_model().clear()
@@ -193,10 +194,10 @@ class DBListEditor(object):
     def profile_create(self, button):
         self.clear_entries()
         model = self.profile_tree.get_model()
-        selection = self.profile_tree.get_selection()
         model.append(['', False])
         column = self.profile_tree.get_column(0)
-        self.profile_tree.set_cursor(len(model)-1, column, start_editing=True)
+        self.profile_tree.set_cursor(len(model) - 1, column,
+            start_editing=True)
 
     def profile_delete(self, button):
         self.clear_entries()
@@ -231,22 +232,13 @@ class DBListEditor(object):
 
         self.display_dbwidget(None, None, self.current_database)
 
-    def edit_canceled(self, renderer):
-        model = self.profile_tree.get_model()
-        for i, row in enumerate(list(model)):
-            if not row[0]:
-                del model[i]
-
-    def check_edit_cancel(self, editable, event, renderer, path):
-        renderer.emit('edited', path, editable.get_text())
-        return False
-
     def edit_started(self, renderer, editable, path):
         if isinstance(editable, gtk.Entry):
-            editable.connect('focus-out-event', self.check_edit_cancel,
+            editable.connect('focus-out-event', self.edit_profilename,
                 renderer, path)
 
-    def edit_profilename(self, renderer, path, newtext):
+    def edit_profilename(self, editable, event, renderer, path):
+        newtext = editable.get_text()
         model = self.profile_tree.get_model()
         oldname = model[path][0]
         if oldname == newtext == '':
@@ -290,6 +282,8 @@ class DBListEditor(object):
         port = self.port_entry.get_text()
         if not (host and port):
             return
+        if (host, port, self.current_profile['name']) == self.db_cache:
+            return
         if self.updating_db:
             return
         if dbname is None:
@@ -307,6 +301,7 @@ class DBListEditor(object):
 
         def callback(dbs, createdb):
             self.updating_db = False
+            self.db_cache = (host, port, self.current_profile['name'])
 
             if dbs is None and createdb is None:
                 pass
@@ -343,6 +338,7 @@ class DBListEditor(object):
         port = int(self.port_entry.get_text())
         dia = DBCreate(host, port)
         dbname = dia.run()
+        self.db_cache = None
         self.username_entry.set_text('admin')
         self.display_dbwidget(None, None, dbname)
 
@@ -375,7 +371,7 @@ class DBLogin(object):
         # GTK Stuffs
         self.parent = common.get_toplevel_window()
         self.dialog = gtk.Dialog(title=_('Login'), parent=self.parent,
-            flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT)
+            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
         self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.dialog.set_has_separator(True)
         self.dialog.set_icon(TRYTON_ICON)
@@ -444,7 +440,8 @@ class DBLogin(object):
         self.label_host.set_alignment(1, 0.5)
         self.label_host.set_padding(3, 3)
         self.entry_host = gtk.Entry()
-        self.entry_host.connect_after('focus-out-event', self.clear_profile_combo)
+        self.entry_host.connect_after('focus-out-event',
+            self.clear_profile_combo)
         self.entry_host.set_activates_default(True)
         self.table_main.attach(self.label_host, 0, 1, 4, 5, xoptions=gtk.FILL)
         self.table_main.attach(self.entry_host, 1, 3, 4, 5)
@@ -453,7 +450,8 @@ class DBLogin(object):
         self.label_database.set_alignment(1, 0.5)
         self.label_database.set_padding(3, 3)
         self.entry_database = gtk.Entry()
-        self.entry_database.connect_after('focus-out-event', self.clear_profile_combo)
+        self.entry_database.connect_after('focus-out-event',
+            self.clear_profile_combo)
         self.entry_database.set_activates_default(True)
         self.table_main.attach(self.label_database, 0, 1, 5, 6,
             xoptions=gtk.FILL)
@@ -465,12 +463,12 @@ class DBLogin(object):
         self.entry_login = gtk.Entry()
         self.entry_login.set_activates_default(True)
         self.table_main.attach(self.entry_login, 1, 3, 6, 7)
-        label_password = gtk.Label(str = _("Password:"))
+        label_password = gtk.Label(str=_("Password:"))
         label_password.set_justify(gtk.JUSTIFY_RIGHT)
         label_password.set_alignment(1, 0.5)
         label_password.set_padding(3, 3)
         self.table_main.attach(label_password, 0, 1, 7, 8, xoptions=gtk.FILL)
-        label_username = gtk.Label(str = _("User name:"))
+        label_username = gtk.Label(str=_("User name:"))
         label_username.set_alignment(1, 0.5)
         label_username.set_padding(3, 3)
         self.table_main.attach(label_username, 0, 1, 6, 7, xoptions=gtk.FILL)
@@ -631,13 +629,10 @@ class DBLogin(object):
                 port = self.get_port(netloc)
             except ValueError:
                 continue
-            if ':' in host:
-                hostname = '[%s]' % host
-            else:
-                hostname = host
             try:
                 if not common.test_server_version(host, port):
-                    common.warning('', _(u'Incompatible version of the server'))
+                    common.warning('',
+                        _(u'Incompatible version of the server'))
                     continue
             except Exception, exception:
                 common.process_exception(exception)
@@ -660,4 +655,3 @@ class DBLogin(object):
             Main.get_main().refresh_ssl()
             raise TrytonError('QueryCanceled')
         return result
-
