@@ -5,24 +5,20 @@ import gettext
 import gtk
 import gobject
 import locale
-import tryton.rpc as rpc
 from tryton.gui.window.view_form.screen import Screen
 from tryton.action import Action
-from tryton.config import CONFIG
 from tryton.gui import Main
 from tryton.gui.window import Window
-from tryton.gui.window.preference import Preference
 from tryton.gui.window.win_export import WinExport
 from tryton.gui.window.win_import import WinImport
 from tryton.gui.window.attachment import Attachment
 from tryton.signal_event import SignalEvent
 from tryton.common import (TRYTON_ICON, message, sur, sur_3b, COLOR_SCHEMES,
     timezoned_date)
-from tryton.exceptions import TrytonServerError
 import tryton.common as common
 from tryton.translate import date_format
-from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT
-from tryton.pyson import PYSONEncoder
+from tryton.common import HM_FORMAT
+from tryton.common import RPCExecute, RPCException
 
 from tabcontent import TabContent
 
@@ -53,30 +49,32 @@ class Form(SignalEvent, TabContent):
     menu_def = [
         (_('_New'), 'tryton-new', 'sig_new', '<tryton>/Form/New'),
         (_('_Save'), 'tryton-save', 'sig_save', '<tryton>/Form/Save'),
+        (_('_Switch View'), 'tryton-fullscreen', 'sig_switch',
+            '<tryton>/Form/Switch View'),
+        (_('_Reload/Undo'), 'tryton-refresh', 'sig_reload',
+            '<tryton>/Form/Reload'),
         (_('_Duplicate'), 'tryton-copy', 'sig_copy',
             '<tryton>/Form/Duplicate'),
         (_('_Delete...'), 'tryton-delete', 'sig_remove',
             '<tryton>/Form/Delete'),
         (None,) * 4,
-        (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
         (_('_Previous'), 'tryton-go-previous', 'sig_previous',
             '<tryton>/Form/Previous'),
-        (_('_Switch View'), 'tryton-fullscreen', 'sig_switch',
-            '<tryton>/Form/Switch View'),
+        (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
+        (_('_Search'), 'tryton-find', 'sig_search', '<tryton>/Form/Search'),
+        (_('View _Logs...'), None, 'sig_logs', None),
+        (_('_Go to Record ID...'), None, 'sig_goto', '<tryton>/Form/Goto'),
         (None,) * 4,
         (_('_Close Tab'), 'tryton-close', 'sig_win_close',
             '<tryton>/Form/Close'),
         (None,) * 4,
-        (_('View _Logs...'), None, 'sig_logs', None),
-        (_('_Go to Record ID...'), None, 'sig_goto', '<tryton>/Form/Goto'),
-        (None,) * 4,
-        (_('_Reload/Undo'), 'tryton-refresh', 'sig_reload',
-            '<tryton>/Form/Reload'),
-        (None,) * 4,
+        (_('A_ttachments...'), 'tryton-attachment', 'sig_attach',
+            '<tryton>/Form/Attachments'),
         (_('_Actions...'), 'tryton-executable', 'sig_action',
             '<tryton>/Form/Actions'),
         (_('_Relate...'), 'tryton-go-jump', 'sig_relate',
             '<tryton>/Form/Relate'),
+        (None,) * 4,
         (_('_Report...'), 'tryton-print-open', 'sig_print_open',
             '<tryton>/Form/Report'),
         (_('_E-Mail...'), 'tryton-print-email', 'sig_print_email',
@@ -127,7 +125,7 @@ class Form(SignalEvent, TabContent):
         self.screen.signal_connect(self, 'record-message',
             self._record_message)
         self.screen.signal_connect(self, 'record-modified',
-            self._record_modified)
+            lambda *a: gobject.idle_add(self._record_modified, *a))
         self.screen.signal_connect(self, 'record-saved', self._record_saved)
         self.screen.signal_connect(self, 'attachment-count',
                 self._attachment_count)
@@ -147,16 +145,11 @@ class Form(SignalEvent, TabContent):
             gobject.timeout_add(int(auto_refresh) * 1000, self.sig_reload)
 
     def get_toolbars(self):
-        ctx = {}
-        ctx.update(rpc.CONTEXT)
-        ctx.update(self.context)
-        args = ('model', self.model, 'view_toolbar_get', ctx)
         try:
-            toolbars = rpc.execute(*args)
-        except TrytonServerError, exception:
-            toolbars = common.process_exception(exception, *args)
-            toolbars = toolbars if toolbars else {}
-        return toolbars
+            return RPCExecute('model', self.model, 'view_toolbar_get',
+                context=self.context)
+        except RPCException:
+            return {}
 
     def widget_get(self):
         return self.screen.widget
@@ -182,9 +175,9 @@ class Form(SignalEvent, TabContent):
             return
         parent = common.get_toplevel_window()
         win = gtk.Dialog(_('Go to ID'), parent,
-                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
-                (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                    gtk.STOCK_OK, gtk.RESPONSE_OK))
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK,
+                gtk.RESPONSE_OK))
         win.set_icon(TRYTON_ICON)
         win.set_has_separator(True)
         win.set_default_response(gtk.RESPONSE_OK)
@@ -199,6 +192,7 @@ class Form(SignalEvent, TabContent):
         entry.set_property('activates_default', True)
         entry.set_max_length(0)
         entry.set_alignment(1.0)
+
         def sig_insert_text(widget, new_text, new_text_length, position):
             value = widget.get_text()
             position = widget.get_position()
@@ -220,7 +214,6 @@ class Form(SignalEvent, TabContent):
         parent.present()
 
     def destroy(self):
-        self.screen.signal_unconnect(self)
         self.screen.destroy()
         self.screen = None
         self.widget = None
@@ -239,7 +232,9 @@ class Form(SignalEvent, TabContent):
     def sig_attach(self, widget=None):
         obj_id = self.id_get()
         if obj_id >= 0 and obj_id is not False:
-            Attachment(self.model, obj_id).run()
+            def callback():
+                self.update_attachment_count(reload=True)
+            Attachment(self.model, obj_id, callback)
         else:
             self.message_info(_('No record selected!'))
         self.update_attachment_count(reload=True)
@@ -280,16 +275,11 @@ class Form(SignalEvent, TabContent):
             ('write_date', _('Latest Modification Date:')),
         ]
 
-        ctx = self.context.copy()
-        ctx.update(rpc.CONTEXT)
-        args = ('model', self.model, 'read', [obj_id], [x[0] for x in fields],
-                ctx)
         try:
-            res = rpc.execute(*args)
-        except TrytonServerError, exception:
-            res = common.process_exception(exception, *args)
-            if not res:
-                return
+            res = RPCExecute('model', self.model, 'read', [obj_id],
+                [x[0] for x in fields], context=self.context)
+        except RPCException:
+            return
         message_str = ''
         for line in res:
             for (key, val) in fields:
@@ -299,7 +289,7 @@ class Form(SignalEvent, TabContent):
                     display_format = date_format() + ' ' + HM_FORMAT
                     date = timezoned_date(line[key])
                     value = common.datetime_strftime(date, display_format)
-                message_str += val + ' ' + value +'\n'
+                message_str += val + ' ' + value + '\n'
         message_str += _('Model:') + ' ' + self.model
         message(message_str)
         return True
@@ -346,17 +336,14 @@ class Form(SignalEvent, TabContent):
         if not self.modified_save():
             return
         res_ids = self.sel_ids_get()
-        ctx = self.context.copy()
-        ctx.update(rpc.CONTEXT)
-        args = ('model', self.model, 'copy', res_ids, {}, ctx)
         try:
-            new_ids = rpc.execute(*args)
-        except TrytonServerError, exception:
-            new_ids = common.process_exception(exception, *args)
-        if new_ids:
-            self.screen.load(new_ids)
-            self.message_info(_('Working now on the duplicated record(s)!'),
-                    'green')
+            new_ids = RPCExecute('model', self.model, 'copy', res_ids, {},
+                context=self.context)
+        except RPCException:
+            return
+        self.screen.load(new_ids)
+        self.message_info(_('Working now on the duplicated record(s)!'),
+            'green')
 
     def sig_save(self, widget=None):
         if self.screen.save_current():
@@ -371,12 +358,14 @@ class Form(SignalEvent, TabContent):
             return
         self.screen.display_prev()
         self.message_info('')
+        self.activate_save()
 
     def sig_next(self, widget=None):
         if not self.modified_save():
             return
         self.screen.display_next()
         self.message_info('')
+        self.activate_save()
 
     def sig_reload(self, test_modified=True):
         if not hasattr(self, 'screen'):
@@ -390,17 +379,17 @@ class Form(SignalEvent, TabContent):
                 pass
             else:
                 return False
-        if self.screen.current_view.view_type == 'form':
-            self.screen.cancel_current()
-            self.screen.display()
-        else:
+        self.screen.cancel_current()
+        set_cursor = False
+        if self.screen.current_view.view_type != 'form':
             obj_id = self.id_get()
             self.screen.search_filter(self.screen.screen_container.get_text())
             for record in self.screen.group:
                 if record.id == obj_id:
                     self.screen.current_record = record
-                    self.screen.display(set_cursor=True)
+                    set_cursor = True
                     break
+        self.screen.display(set_cursor=set_cursor)
         self.message_info('')
         self.activate_save()
         return True
@@ -420,6 +409,11 @@ class Form(SignalEvent, TabContent):
     def sig_relate(self, widget):
         self.buttons['relate'].props.active = True
 
+    def sig_search(self, widget):
+        search_container = self.screen.screen_container
+        if hasattr(search_container, 'search_entry'):
+            search_container.search_entry.grab_focus()
+
     def action_popup(self, widget):
         button, = widget.get_children()
         button.grab_focus()
@@ -469,31 +463,31 @@ class Form(SignalEvent, TabContent):
         self.activate_save()
 
     def _record_modified(self, screen, signal_data):
-        self.message_info('', color='white')
-        self.activate_save()
+        # As it is called via idle_add, the form could have been destroyed in
+        # the meantime.
+        if screen == self.screen:
+            self.activate_save()
 
     def _record_saved(self, screen, signal_data):
         self.activate_save()
 
-    def modified_save(self, reload=True):
+    def modified_save(self):
+        self.screen.current_view.set_value()
         if self.screen.modified():
             value = sur_3b(
                     _('This record has been modified\n' \
                             'do you want to save it ?'))
             if value == 'ok':
                 return self.sig_save(None)
-            elif value == 'ko':
-                if reload:
-                    self.sig_reload(test_modified=False)
-                return True
-            else:
-                return False
+            if value == 'ko':
+                return self.sig_reload(test_modified=False)
+            return False
         return True
 
     def sig_close(self, widget=None):
         for dialog in self.dialogs[:]:
             dialog.destroy()
-        return self.modified_save(reload=False)
+        return self.modified_save()
 
     def _action(self, action, atype):
         action = action.copy()
@@ -501,30 +495,7 @@ class Form(SignalEvent, TabContent):
             return
         record_id = self.screen.id_get()
         record_ids = self.screen.sel_ids_get()
-        if atype in ('print', 'action'):
-            email = {}
-            if 'pyson_email' in action:
-                email = self.screen.current_record.expr_eval(
-                    action['pyson_email'])
-                if not email:
-                    email = {}
-            if 'subject' not in email:
-                email['subject'] = action['name'].replace('_', '')
-            action['email'] = email
-            self.screen.display()
-        elif atype == 'relate':
-            encoder = PYSONEncoder()
-            if 'pyson_domain' in action:
-                action['pyson_domain'] = encoder.encode(
-                        self.screen.current_record.expr_eval(
-                                action['pyson_domain'], check_load=False))
-            if 'pyson_context' in action:
-                action['pyson_context'] = encoder.encode(
-                        self.screen.current_record.expr_eval(
-                            action['pyson_context'], check_load=False))
-        else:
-            raise NotImplementedError("Action type '%s' is not supported" %
-                atype)
+        action = Action.evaluate(action, atype, self.screen.current_record)
         data = {
             'model': self.screen.model_name,
             'id': record_id,
@@ -604,3 +575,7 @@ class Form(SignalEvent, TabContent):
 
     def _popup_menu_hide(self, menuitem, togglebutton):
         togglebutton.props.active = False
+
+    def set_cursor(self):
+        if self.screen:
+            self.screen.set_cursor()
