@@ -1,5 +1,7 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+import gtk
+
 from tryton.gui.main import Main
 import tryton.common as common
 
@@ -10,7 +12,11 @@ class NoModal(object):
         self.parent = common.get_toplevel_window()
         self.sensible_widget = common.get_sensible_widget(self.parent)
         self.page = None
-        self.parent_focus = self.parent.get_focus()
+        self.parent_focus = []
+        focus = self.parent.get_focus()
+        while focus:
+            self.parent_focus.append(focus)
+            focus = focus.get_parent()
 
     def register(self):
         main = Main.get_main()
@@ -22,7 +28,12 @@ class NoModal(object):
 
     def destroy(self):
         self.page.dialogs.remove(self)
+        # Test if the parent is not already destroyed
+        if self.parent not in gtk.window_list_toplevels():
+            return
         self.parent.present()
         self.sensible_widget.props.sensitive = True
-        if self.parent_focus:
-            self.parent_focus.grab_focus()
+        for focus in self.parent_focus:
+            if focus and focus.is_ancestor(self.parent):
+                focus.grab_focus()
+                break
