@@ -2,16 +2,17 @@
 #this repository contains the full copyright notices and license terms.
 import os
 import tempfile
-import tryton.rpc as rpc
-from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT, datetime_strftime, \
+import locale
+from tryton.common import datetime_strftime, HM_FORMAT, \
         domain_inversion, eval_domain, localize_domain, unlocalize_domain, \
-        merge, inverse_leaf, EvalEnvironment, RPCProgress
+        merge, inverse_leaf, EvalEnvironment
 import tryton.common as common
 import time
 import datetime
+import decimal
 from decimal import Decimal
-import logging
-from tryton.exceptions import TrytonServerError
+from tryton.translate import date_format
+from tryton.common import RPCExecute, RPCException
 
 
 class Field(object):
@@ -29,6 +30,8 @@ class Field(object):
 
 class CharField(object):
 
+    _default = ''
+
     def __init__(self, attrs):
         self.attrs = attrs
         self.name = attrs['name']
@@ -60,8 +63,8 @@ class CharField(object):
     def validation_domains(self, record):
         screen_domain, attr_domain = self.domains_get(record)
         if attr_domain:
-            return screen_domain, screen_domain + unlocalize_domain(attr_domain,
-                self.name)
+            return (screen_domain, screen_domain +
+                unlocalize_domain(attr_domain, self.name))
         else:
             return screen_domain, screen_domain
 
@@ -74,6 +77,14 @@ class CharField(object):
                 check_load=check_load))
         return context
 
+    def check_required(self, record):
+        state_attrs = self.get_state_attrs(record)
+        if bool(int(state_attrs.get('required') or 0)):
+            if (not self.get(record)
+                    and not bool(int(state_attrs.get('readonly') or 0))):
+                return False
+        return True
+
     def validate(self, record, softvalidation=False):
         if self.attrs.get('readonly'):
             return True
@@ -81,23 +92,23 @@ class CharField(object):
         self.get_state_attrs(record)['domain_readonly'] = False
         inverted_domain, domain = self.validation_domains(record)
         if not softvalidation:
-            if bool(int(self.get_state_attrs(record).get('required') or 0)):
-                if not self.get(record) \
-                        and not bool(int(self.get_state_attrs(record
-                            ).get('readonly') or 0)):
-                    res = False
+            res = res and self.check_required(record)
         if isinstance(domain, bool):
             res = res and domain
         elif domain == [('id', '=', False)]:
             res = False
         else:
-            if (isinstance(inverted_domain, list) \
-                and len(inverted_domain) == 1 and inverted_domain[0][1] == '='):
-                # If the inverted domain is so constraint that only one value is
-                # possible we should use it. But we must also pay attention to
-                # the fact that the original domain might be a 'OR' domain and
-                # thus not preventing the modification of fields.
+            if (isinstance(inverted_domain, list)
+                    and len(inverted_domain) == 1
+                    and inverted_domain[0][1] == '='):
+                # If the inverted domain is so constraint that only one value
+                # is possible we should use it. But we must also pay attention
+                # to the fact that the original domain might be a 'OR' domain
+                # and thus not preventing the modification of fields.
                 leftpart, _, value = inverted_domain[0][:3]
+                if value is False:
+                    # XXX to remove once server domains are fixed
+                    value = None
                 setdefault = True
                 if '.' in leftpart:
                     recordpart, localpart = leftpart.split('.', 1)
@@ -124,7 +135,7 @@ class CharField(object):
         return True
 
     def get(self, record, check_load=True, readonly=True, modified=False):
-        return record.value.get(self.name, False) or False
+        return record.value.get(self.name) or self._default
 
     def get_eval(self, record, check_load=True):
         return self.get(record, check_load=check_load, readonly=True,
@@ -134,18 +145,21 @@ class CharField(object):
         return self.get_eval(record, check_load=check_load)
 
     def set_client(self, record, value, force_change=False):
-        internal = record.value.get(self.name, False)
-        prev_modified_fields = record.modified_fields.copy()
+        previous_value = self.get(record)
         self.set(record, value)
-        if (internal or False) != (record.value.get(self.name, False) or False):
+        if previous_value != self.get(record):
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
             self.sig_changed(record)
             record.validate(softvalidation=True)
             record.signal('record-changed')
+        elif force_change:
+            self.sig_changed(record)
+            record.validate(softvalidation=True)
+            record.signal('record-changed')
 
     def get_client(self, record):
-        return record.value.get(self.name) or False
+        return record.value.get(self.name) or self._default
 
     def set_default(self, record, value, modified=False):
         res = self.set(record, value, modified=modified)
@@ -155,9 +169,6 @@ class CharField(object):
         record.modified_fields.setdefault(self.name)
         return self.set(record, value, modified=False)
 
-    def get_default(self, record):
-        return self.get(record)
-
     def state_set(self, record, states=('readonly', 'required', 'invisible')):
         state_changes = record.expr_eval(self.attrs.get('states', {}),
                 check_load=False)
@@ -189,103 +200,166 @@ class CharField(object):
 
 class SelectionField(CharField):
 
-    def set(self, record, value, modified=False):
-        if isinstance(value, (list, tuple)):
-            value = value[0]
-        return super(SelectionField, self).set(record, value,
-                modified=modified)
+    _default = None
+
+    def get_client(self, record):
+        return record.value.get(self.name)
 
 
 class DateTimeField(CharField):
 
+    _default = None
+
     def set_client(self, record, value, force_change=False):
-        if value and not isinstance(value, datetime.datetime):
-            value = datetime.datetime(*time.strptime(value, DHM_FORMAT)[:6])
-        return super(DateTimeField, self).set_client(record, value,
-                force_change=force_change)
+        if not isinstance(value, datetime.datetime):
+            try:
+                value = datetime.datetime(*time.strptime(value,
+                        date_format() + ' ' + HM_FORMAT)[:6])
+                value = common.untimezoned_date(value)
+            except ValueError:
+                value = self._default
+        super(DateTimeField, self).set_client(record, value,
+            force_change=force_change)
 
     def get_client(self, record):
         value = super(DateTimeField, self).get_client(record)
-        if not value:
-            return False
-        return datetime_strftime(value, DHM_FORMAT)
+        if value:
+            value = common.timezoned_date(value)
+            return datetime_strftime(value, date_format() + ' ' + HM_FORMAT)
+        return ''
 
 
 class DateField(CharField):
 
+    _default = None
+
     def set_client(self, record, value, force_change=False):
-        if value and not isinstance(value, datetime.date):
-            value = datetime.date(*time.strptime(value, DT_FORMAT)[:3])
-        return super(DateField, self).set_client(record, value,
-                force_change=force_change)
+        if not isinstance(value, datetime.date):
+            try:
+                value = datetime.date(*time.strptime(value,
+                        date_format())[:3])
+            except ValueError:
+                value = self._default
+        super(DateField, self).set_client(record, value,
+            force_change=force_change)
 
     def get_client(self, record):
         value = super(DateField, self).get_client(record)
-        if not value:
-            return False
-        return datetime_strftime(value, DT_FORMAT)
+        if value:
+            return datetime_strftime(value, date_format())
+        return ''
 
 
-class FloatField(CharField):
+class TimeField(CharField):
 
     def set_client(self, record, value, force_change=False):
-        internal = record.value.get(self.name)
-        prev_modified_fields = record.modified_fields.copy()
-        self.set(record, value)
-        digits = record.expr_eval(self.attrs.get('digits', (16, 2)))
-        if abs(float(internal or 0.0) - float(record.value[self.name] or 0.0)) \
-                >= (10.0**(-int(digits[1]))):
-            if not self.get_state_attrs(record).get('readonly', False):
-                record.modified_fields.setdefault(self.name)
-                record.signal('record-modified')
-                self.sig_changed(record)
-                record.validate(softvalidation=True)
-                record.signal('record-changed')
-
-
-class NumericField(CharField):
-
-    def set_client(self, record, value, force_change=False):
-        value = Decimal(str(value))
-        internal = record.value.get(self.name)
-        prev_modified_fields = record.modified_fields.copy()
-        self.set(record, value)
-        digits = record.expr_eval(self.attrs.get('digits', (16, 2)))
-        if abs((internal or Decimal('0.0')) - \
-                (record.value[self.name] or Decimal('0.0'))) \
-                >= Decimal(str(10.0**(-int(digits[1])))):
-            if not self.get_state_attrs(record).get('readonly', False):
-                record.modified_fields.setdefault(self.name)
-                record.signal('record-modified')
-                self.sig_changed(record)
-                record.validate(softvalidation=True)
-                record.signal('record-changed')
-
-
-class IntegerField(CharField):
-
-    def get(self, record, check_load=True, readonly=True, modified=False):
-        return record.value.get(self.name) or 0
+        if not isinstance(value, datetime.time):
+            try:
+                value = datetime.time(*time.strptime(value, HM_FORMAT)[3:6])
+            except ValueError:
+                value = None
+        super(TimeField, self).set_client(record, value,
+            force_change=force_change)
 
     def get_client(self, record):
-        return record.value.get(self.name) or 0
+        value = super(TimeField, self).get_client(record)
+        if value:
+            return value.strftime(HM_FORMAT)
+        return ''
+
+
+class NumberField(CharField):
+    _default = None
+
+    def check_required(self, record):
+        state_attrs = self.get_state_attrs(record)
+        if bool(int(state_attrs.get('required') or 0)):
+            if (self.get(record) is None
+                    and not bool(int(state_attrs.get('readonly') or 0))):
+                return False
+        return True
+
+    def get(self, record, check_load=True, readonly=True, modified=False):
+        return record.value.get(self.name, self._default)
+
+    def digits(self, record):
+        default = (16, 2)
+        return tuple(y if x is None else x for x, y in zip(
+                record.expr_eval(self.attrs.get('digits', default)), default))
+
+
+class FloatField(NumberField):
+
+    def set_client(self, record, value, force_change=False):
+        if isinstance(value, basestring):
+            try:
+                value = locale.atof(value)
+            except ValueError:
+                value = self._default
+        super(FloatField, self).set_client(record, value,
+            force_change=force_change)
+
+    def get_client(self, record):
+        value = record.value.get(self.name)
+        if value is not None:
+            digits = self.digits(record)
+            return locale.format('%.' + str(digits[1]) + 'f', value, True)
+        else:
+            return ''
+
+
+class NumericField(NumberField):
+
+    def set_client(self, record, value, force_change=False):
+        if isinstance(value, basestring):
+            try:
+                value = locale.atof(value, Decimal)
+            except decimal.InvalidOperation:
+                value = self._default
+        super(NumericField, self).set_client(record, value,
+            force_change=force_change)
+
+    def get_client(self, record):
+        value = record.value.get(self.name)
+        if value is not None:
+            digits = self.digits(record)
+            return locale.format('%.' + str(digits[1]) + 'f', value, True)
+        else:
+            return ''
+
+
+class IntegerField(NumberField):
+
+    def set_client(self, record, value):
+        if isinstance(value, basestring):
+            try:
+                value = locale.atoi(value)
+            except ValueError:
+                value = self._default
+        super(IntegerField, self).set_client(record, value)
+
+    def get_client(self, record):
+        value = record.value.get(self.name)
+        if value is not None:
+            return locale.format('%d', value, True)
+        else:
+            return ''
+
+    def digits(self, record):
+        return (16, 0)
+
 
 class BooleanField(CharField):
 
+    _default = False
+
     def set_client(self, record, value, force_change=False):
         value = bool(value)
-        internal = bool(record.value.get(self.name, False))
-        prev_modified_fields = record.modified_fields.copy()
-        self.set(record, value)
-        if internal != bool(record.value.get(self.name, False)):
-            record.modified_fields.setdefault(self.name)
-            record.signal('record-modified')
-            self.sig_changed(record)
-            record.validate(softvalidation=True)
-            record.signal('record-changed')
+        super(BooleanField, self).set_client(record, value,
+            force_change=force_change)
 
     def get(self, record, check_load=True, readonly=True, modified=False):
-        return bool(record.value.get(self.name, False))
+        return bool(record.value.get(self.name))
 
     def get_client(self, record):
         return bool(record.value.get(self.name))
@@ -296,90 +370,60 @@ class M2OField(CharField):
     internal = (id, name)
     '''
 
+    _default = None
+
     def get(self, record, check_load=True, readonly=True, modified=False):
         value = record.value.get(self.name)
         if (record.parent_name == self.name
                 and self.attrs['relation'] == record.group.parent.model_name):
-            value = record.parent.id if record.parent else False
-        if value:
-            if isinstance(value, (int, basestring, long)):
-                self.set(record, value)
-                value = record.value.get(self.name, value)
-            if isinstance(value, (int, basestring, long)):
-                return value
-            return value[0] or False
-        return False
+            value = record.parent.id if record.parent else None
+        return value
 
     def get_client(self, record):
-        value = record.value.get(self.name)
-        if (record.parent_name == self.name
-                and self.attrs['relation'] == record.group.parent.model_name):
-            value = record.parent.id if record.parent else False
-        if value:
-            if isinstance(value, (int, basestring, long)):
-                self.set(record, value)
-                value = record.value.get(self.name, value)
-            if isinstance(value, (int, basestring, long)):
-                return value
-            return value[1]
-        return False
+        rec_name = record.value.get(self.name + '.rec_name')
+        if rec_name is None:
+            self.set(record, self.get(record))
+            rec_name = record.value.get(self.name + '.rec_name') or ''
+        return rec_name
+
+    def set_client(self, record, value, force_change=False):
+        if isinstance(value, (tuple, list)):
+            value, rec_name = value
+            record.value[self.name + '.rec_name'] = rec_name
+        super(M2OField, self).set_client(record, value,
+            force_change=force_change)
 
     def set(self, record, value, modified=False):
+        rec_name = record.value.get(self.name + '.rec_name') or ''
+        if value is False:
+            value = None
         if (record.parent_name == self.name
                 and self.attrs['relation'] == record.group.parent.model_name):
             if record.parent:
+                value = record.parent.id
                 if 'rec_name' in record.parent.value:
-                    value = (record.parent.id, record.parent.value['rec_name'])
-                else:
-                    value = record.parent.id
+                    rec_name = record.parent.value['rec_name'] or ''
             else:
-                value = False
-        if value and isinstance(value, (int, long)) and value >= 0:
-            args = ('model', self.attrs['relation'], 'read', value,
-                    ['rec_name'], rpc.CONTEXT)
+                value = None
+        if not rec_name and value >= 0:
             try:
-                result = rpc.execute(*args)
-            except TrytonServerError, exception:
-                result = common.process_exception(exception, *args)
-                if not result:
-                    return
-            value = value, result['rec_name']
-        if value and (isinstance(value, (int, long))
-                or len(value) != 2):
-            value = (False, '')
-            record.value[self.name + '.rec_name'] = ''
-        else:
-            if value:
-                record.value[self.name + '.rec_name'] = value[1]
-            else:
-                record.value[self.name + '.rec_name'] = ''
-        record.value[self.name] = value or (False, '')
+                result = RPCExecute('model', self.attrs['relation'], 'read',
+                    value, ['rec_name'])
+            except RPCException:
+                return False
+            rec_name = result['rec_name'] or ''
+        record.value[self.name + '.rec_name'] = rec_name
+        record.value[self.name] = value
         if (record.parent_name == self.name
                 and self.attrs['relation'] == record.group.parent.model_name):
             if record.parent:
                 if 'rec_name' not in record.parent.value:
-                    record.parent.value['rec_name'] = \
-                            record.value[self.name + '.rec_name']
+                    record.parent.value['rec_name'] = rec_name
         if modified:
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
             record.signal('record-changed')
 
-    def set_client(self, record, value, force_change=False):
-        internal = record.value.get(self.name) or (False, '')
-        prev_modified_fields = record.modified_fields.copy()
-        self.set(record, value)
-        if (internal[0] or False) != (record.value[self.name][0] or False):
-            record.modified_fields.setdefault(self.name)
-            record.signal('record-modified')
-            self.sig_changed(record)
-            record.validate(softvalidation=True)
-            record.signal('record-changed')
-        elif force_change:
-            self.sig_changed(record)
-            record.validate(softvalidation=True)
-            record.signal('record-changed')
-
     def context_get(self, record, check_load=True, eval_context=True):
         context = super(M2OField, self).context_get(record,
                 check_load=check_load, eval_context=eval_context)
@@ -394,7 +438,8 @@ class M2OField(CharField):
 
     def domain_get(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        return localize_domain(inverse_leaf(screen_domain), self.name) + attr_domain
+        return (localize_domain(inverse_leaf(screen_domain), self.name)
+            + attr_domain)
 
     def get_state_attrs(self, record):
         result = super(M2OField, self).get_state_attrs(record)
@@ -404,6 +449,7 @@ class M2OField(CharField):
             result['readonly'] = True
         return result
 
+
 class O2OField(M2OField):
     pass
 
@@ -413,6 +459,8 @@ class O2MField(CharField):
     internal = Group of the related objects
     '''
 
+    _default = None
+
     def __init__(self, attrs):
         super(O2MField, self).__init__(attrs)
         self.in_on_change = False
@@ -426,7 +474,6 @@ class O2MField(CharField):
         if not record.parent:
             return
         record.parent.modified_fields.setdefault(self.name)
-        record.parent.signal('record-modified')
         self.sig_changed(record.parent)
         record.parent.validate(softvalidation=True)
         record.parent.signal('record-changed')
@@ -439,6 +486,11 @@ class O2MField(CharField):
         if group.model_name == group.parent.model_name:
             group.parent.signal('group-cleared')
 
+    def _record_modified(self, group, record):
+        if not record.parent:
+            return
+        record.parent.signal('record-modified')
+
     def _set_default_value(self, record):
         if record.value.get(self.name) is not None:
             return
@@ -453,8 +505,10 @@ class O2MField(CharField):
         if record.model_name == self.attrs['relation']:
             group.fields = record.group.fields
         group.signal_connect(group, 'group-changed', self._group_changed)
-        group.signal_connect(group, 'group-list-changed', self._group_list_changed)
+        group.signal_connect(group, 'group-list-changed',
+            self._group_list_changed)
         group.signal_connect(group, 'group-cleared', self._group_cleared)
+        group.signal_connect(group, 'record-modified', self._record_modified)
         record.value[self.name] = group
 
     def get_client(self, record):
@@ -483,6 +537,8 @@ class O2MField(CharField):
                     get_readonly=readonly)
                 values.pop(parent_name, None)
                 result.append(('create', values))
+        if not result[0][1]:
+            del result[0]
         if record_removed:
             result.append(('unlink', [x.id for x in record_removed]))
         if record_deleted:
@@ -493,14 +549,19 @@ class O2MField(CharField):
         if record.value.get(self.name) is None:
             return {}
         result = {}
-        for record2 in (record.value[self.name] \
-                + record.value[self.name].record_removed \
+        for record2 in (record.value[self.name]
+                + record.value[self.name].record_removed
                 + record.value[self.name].record_deleted):
-             result.update(record2.get_timestamp())
+            result.update(record2.get_timestamp())
         return result
 
     def get_eval(self, record, check_load=True):
-        return [x.id for x in record.value.get(self.name) or []]
+        if record.value.get(self.name) is None:
+            return []
+        record_removed = record.value[self.name].record_removed
+        record_deleted = record.value[self.name].record_deleted
+        return [x.id for x in record.value[self.name]
+            if x not in record_removed and x not in record_deleted]
 
     def get_on_change_value(self, record, check_load=True):
         result = []
@@ -516,8 +577,10 @@ class O2MField(CharField):
         group = record.value.get(self.name)
         fields = {}
         if group is not None:
+            fields = group.fields.copy()
+            # Unconnect to prevent infinite loop
             group.signal_unconnect(group)
-            fields = group.fields
+            group.destroy()
         elif record.model_name == self.attrs['relation']:
             fields = record.group.fields
         parent_name = self.attrs.get('relation_field', '')
@@ -530,8 +593,10 @@ class O2MField(CharField):
         record.value[self.name] = group
         group.load(value, display=False)
         group.signal_connect(group, 'group-changed', self._group_changed)
-        group.signal_connect(group, 'group-list-changed', self._group_list_changed)
+        group.signal_connect(group, 'group-list-changed',
+            self._group_list_changed)
         group.signal_connect(group, 'group-cleared', self._group_cleared)
+        group.signal_connect(group, 'record-modified', self._record_modified)
         if modified:
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
@@ -550,8 +615,10 @@ class O2MField(CharField):
         group = record.value.get(self.name)
         fields = {}
         if group is not None:
+            fields = group.fields.copy()
+            # Unconnect to prevent infinite loop
             group.signal_unconnect(group)
-            fields = group.fields
+            group.destroy()
         elif record.model_name == self.attrs['relation']:
             fields = record.group.fields
         if fields:
@@ -566,14 +633,11 @@ class O2MField(CharField):
                             and fieldname not in fields):
                         field_names.append(fieldname)
             if field_names:
-                args = ('model', self.attrs['relation'], 'fields_get',
-                        field_names, context)
                 try:
-                    fields.update(rpc.execute(*args))
-                except TrytonServerError, exception:
-                    fields = common.process_exception(exception, *args)
-                    if not fields:
-                        return False
+                    fields.update(RPCExecute('model', self.attrs['relation'],
+                            'fields_get', field_names, context=context))
+                except RPCException:
+                    return False
 
         parent_name = self.attrs.get('relation_field', '')
         group = Group(self.attrs['relation'], fields,
@@ -591,14 +655,16 @@ class O2MField(CharField):
             new_record.set_default(vals, modified=modified)
             group.add(new_record)
         group.signal_connect(group, 'group-changed', self._group_changed)
-        group.signal_connect(group, 'group-list-changed', self._group_list_changed)
+        group.signal_connect(group, 'group-list-changed',
+            self._group_list_changed)
         group.signal_connect(group, 'group-cleared', self._group_cleared)
+        group.signal_connect(group, 'record-modified', self._record_modified)
         return True
 
     def set_on_change(self, record, value):
         self._set_default_value(record)
         if isinstance(value, (list, tuple)):
-            return self.set(record, value, modified=True)
+            return self.set(record, value, modified=False)
 
         if value and (value.get('add') or value.get('update')):
             context = self.context_get(record)
@@ -607,14 +673,11 @@ class O2MField(CharField):
                 for fieldname in val.keys():
                     if fieldname not in field_names:
                         field_names.append(fieldname)
-            args = ('model', self.attrs['relation'], 'fields_get',
-                    field_names, context)
             try:
-                fields = rpc.execute(*args)
-            except TrytonServerError, exception:
-                fields = common.process_exception(exception, *args)
-                if not fields:
-                    return False
+                fields = RPCExecute('model', self.attrs['relation'],
+                    'fields_get', field_names, context=context)
+            except RPCException:
+                return False
 
         to_remove = []
         for record2 in record.value[self.name]:
@@ -632,8 +695,7 @@ class O2MField(CharField):
         if value and value.get('add') or value.get('update', []):
             record.value[self.name].add_fields(fields, signal=False)
             for vals in value.get('add', []):
-                new_record = record.value[self.name].new(default=False,
-                        signal=False)
+                new_record = record.value[self.name].new(default=False)
                 record.value[self.name].add(new_record)
                 new_record.set(vals, modified=True, signal=False)
 
@@ -645,10 +707,6 @@ class O2MField(CharField):
                     record2.set(vals, modified=True, signal=False)
         return True
 
-    def get_default(self, record):
-        res = [x.get_default() for x in record.value.get(self.name) or []]
-        return res
-
     def validation_domains(self, record):
         screen_domain, attr_domain = self.domains_get(record)
         return screen_domain, screen_domain
@@ -688,19 +746,15 @@ class O2MField(CharField):
 
 class M2MField(O2MField):
 
-    def get_default(self, record):
-        return [x.id for x in record.value.get(self.name) or [] if x.id >= 0]
-
-    def get_eval(self, record, check_load=True):
-        return [x.id for x in record.value.get(self.name) or []]
-
     def set(self, record, value, modified=False):
         from group import Group
         group = record.value.get(self.name)
         fields = {}
         if group is not None:
+            fields = group.fields.copy()
+            # Unconnect to prevent infinite loop
             group.signal_unconnect(group)
-            fields = group.fields
+            group.destroy()
         elif record.model_name == self.attrs['relation']:
             fields = record.group.fields
         parent_name = self.attrs.get('relation_field', '')
@@ -717,8 +771,10 @@ class M2MField(O2MField):
         group.fields = fields
         group.load(value, display=False)
         group.signal_connect(group, 'group-changed', self._group_changed)
-        group.signal_connect(group, 'group-list-changed', self._group_list_changed)
+        group.signal_connect(group, 'group-list-changed',
+            self._group_list_changed)
         group.signal_connect(group, 'group-cleared', self._group_cleared)
+        group.signal_connect(group, 'record-modified', self._record_modified)
         if modified:
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
@@ -727,75 +783,72 @@ class M2MField(O2MField):
 
 class ReferenceField(CharField):
 
+    _default = None
+
     def get_client(self, record):
         if record.value.get(self.name):
-            return record.value[self.name]
-        return False
+            model, _ = record.value[self.name]
+            name = record.value.get(self.name + '.rec_name') or ''
+            return model, name
+        else:
+            return None
 
     def get(self, record, check_load=True, readonly=True, modified=False):
         if record.value.get(self.name):
-            if isinstance(record.value[self.name][1], (list, tuple)):
-                return '%s,%s' % (record.value[self.name][0],
-                        str(record.value[self.name][1][0]))
-            else:
-                return '%s,%s' % (record.value[self.name][0],
-                        str(record.value[self.name][1]))
-        return False
+            return ','.join(map(str, record.value[self.name]))
+        return None
 
     def set_client(self, record, value, force_change=False):
-        internal = record.value.get(self.name)
-        prev_modified_fields = record.modified_fields.copy()
-        self.set(record, value)
-        if (internal or False) != (record.value[self.name] or False):
-            record.modified_fields.setdefault(self.name)
-            record.signal('record-modified')
-            self.sig_changed(record)
-            record.validate(softvalidation=True)
-            record.signal('record-changed')
+        if value:
+            ref_model, ref_id = value
+            if isinstance(ref_id, (tuple, list)):
+                ref_id, rec_name = ref_id
+                record.value[self.name + '.rec_name'] = rec_name
+            value = (ref_model, ref_id)
+        super(ReferenceField, self).set_client(record, value,
+            force_change=force_change)
 
     def set(self, record, value, modified=False):
         if not value:
-            record.value[self.name] = False
+            record.value[self.name] = self._default
             return
         if isinstance(value, basestring):
-            model, ref_id = value.split(',')
-            value = model, (ref_id, record.value.get(self.name + '.rec_name'))
-        ref_model, (ref_id, ref_str) = value
-        if ref_model:
-            ref_id = int(ref_id)
-            if ref_id < 0:
-                ref_str = ''
-            if not ref_str and ref_id >= 0:
-                args = ('model', ref_model, 'read', ref_id,
-                        ['rec_name'], rpc.CONTEXT)
-                try:
-                    result = rpc.execute(*args)
-                except TrytonServerError, exception:
-                    result = common.process_exception(exception, *args)
-                    if not result:
-                        return
-                result = result['rec_name']
-                if result:
-                    record.value[self.name] = ref_model, (ref_id, result)
-                    record.value[self.name + '.rec_name'] = result
-                else:
-                    record.value[self.name] = ref_model, (-1, '')
-                    record.value[self.name + '.rec_name'] = ''
+            ref_model, ref_id = value.split(',')
+            if not ref_id:
+                ref_id = None
             else:
-                record.value[self.name] = ref_model, (ref_id, ref_str)
-                record.value[self.name + '.rec_name'] = ref_str
+                try:
+                    ref_id = int(ref_id)
+                except ValueError:
+                    pass
         else:
-            record.value[self.name] = ref_model, (ref_id, ref_id)
-            if self.name + '.rec_name' in record.value:
-                del record.value[self.name + '.rec_name']
+            ref_model, ref_id = value
+        rec_name = record.value.get(self.name + '.rec_name') or ''
+        if ref_model and ref_id >= 0:
+            if not rec_name and ref_id >= 0:
+                try:
+                    result = RPCExecute('model', ref_model, 'read', ref_id,
+                        ['rec_name'])
+                except RPCException:
+                    return
+                rec_name = result['rec_name']
+        elif ref_model:
+            rec_name = ''
+        else:
+            rec_name = ref_id
+        record.value[self.name] = ref_model, ref_id
+        record.value[self.name + '.rec_name'] = rec_name
         if modified:
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
 
+
 class BinaryField(CharField):
 
+    _default = None
+
     def get(self, record, check_load=True, readonly=True, modified=False):
-        result = record.value.get(self.name, False) or False
+        result = record.value.get(self.name) or self._default
         if isinstance(result, basestring):
             try:
                 with open(result, 'rb') as fp:
@@ -830,14 +883,11 @@ class BinaryField(CharField):
         if not isinstance(record.value.get(self.name), (basestring, buffer)):
             if record.id < 0:
                 return ''
-            ctx = rpc.CONTEXT.copy()
-            ctx.update(record.context_get())
-            rpcprogress = RPCProgress('execute', ('model', record.model_name,
-                    'read', record.id, [self.name], ctx))
+            context = record.context_get()
             try:
-                values = rpcprogress.run()
-            except TrytonServerError, exception:
-                values = common.process_exception(exception)
+                values = RPCExecute('model', record.model_name, 'read',
+                    record.id, [self.name], context=context)
+            except RPCException:
                 return ''
             _, filename = tempfile.mkstemp(prefix='tryton_')
             with open(filename, 'wb') as fp:
@@ -849,18 +899,19 @@ TYPES = {
     'char': CharField,
     'sha': CharField,
     'float_time': FloatField,
-    'integer' : IntegerField,
-    'biginteger' : IntegerField,
-    'float' : FloatField,
-    'numeric' : NumericField,
-    'many2one' : M2OField,
-    'many2many' : M2MField,
-    'one2many' : O2MField,
-    'reference' : ReferenceField,
+    'integer': IntegerField,
+    'biginteger': IntegerField,
+    'float': FloatField,
+    'numeric': NumericField,
+    'many2one': M2OField,
+    'many2many': M2MField,
+    'one2many': O2MField,
+    'reference': ReferenceField,
     'selection': SelectionField,
     'boolean': BooleanField,
     'datetime': DateTimeField,
     'date': DateField,
+    'time': TimeField,
     'one2one': O2OField,
     'binary': BinaryField,
 }
