@@ -1,24 +1,32 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+try:
+    from weakref import WeakSet
+except ImportError:
+    from weakrefset import WeakSet
+from collections import defaultdict
 import tryton.rpc as rpc
 from tryton.signal_event import SignalEvent
 import tryton.common as common
 from tryton.pyson import PYSONDecoder
-import field
-import datetime
-import logging
-import time
+import field as fields
+from functools import reduce
+from tryton.common import RPCExecute, RPCException
+
+POOL = defaultdict(WeakSet)
 
 
 class Record(SignalEvent):
 
     id = -1
 
-    def __init__(self, model_name, obj_id, window, group=None):
+    def __init__(self, model_name, obj_id, group=None):
         super(Record, self).__init__()
-        self.__window = window
         self.model_name = model_name
-        self.id = obj_id or Record.id
+        if obj_id is None:
+            self.id = Record.id
+        else:
+            self.id = obj_id
         if self.id < 0:
             Record.id -= 1
         self._loaded = set()
@@ -29,13 +37,18 @@ class Record(SignalEvent):
         self.modified_fields = {}
         self._timestamp = None
         self.attachment_count = -1
-        self.next = {} # Used in Group list
+        self.next = {}  # Used in Group list
         self.value = {}
         self.autocompletion = {}
+        self.exception = False
+        self.destroyed = False
+        POOL[model_name].add(self)
 
-    def __getitem__(self, name, raise_exception=False):
-        if name not in self._loaded and self.id > 0:
-            ids =  [self.id]
+    def __getitem__(self, name):
+        if name not in self._loaded and self.id >= 0:
+            id2record = {
+                self.id: self,
+                }
             if name == '*':
                 loading = reduce(
                         lambda x, y: 'eager' if x == y == 'eager' else 'lazy',
@@ -48,44 +61,62 @@ class Record(SignalEvent):
                 idx = self.group.index(self)
                 length = len(self.group)
                 n = 1
-                while len(ids) < 80 and (idx - n >= 0 or \
+                while len(id2record) < 80 and (idx - n >= 0 or \
                         idx + n < length) and n < 100:
                     if idx - n >= 0:
                         record = self.group[idx - n]
-                        if name not in record._loaded and record.id > 0:
-                            ids.append(record.id)
+                        if name not in record._loaded and record.id >= 0:
+                            id2record[record.id] = record
                     if idx + n < length:
                         record = self.group[idx + n]
-                        if name not in record._loaded and record.id > 0:
-                            ids.append(record.id)
+                        if name not in record._loaded and record.id >= 0:
+                            id2record[record.id] = record
                     n += 1
+            record_context = self.context_get()
+            if loading == 'eager' and len(id2record) < 80:
+                for record in POOL[self.model_name]:
+                    if (name not in record._loaded
+                            and record.id >= 0
+                            and record.id not in id2record
+                            and record.context_get() == record_context):
+                        id2record[record.id] = record
+                        if len(id2record) == 80:
+                            break
             if loading == 'eager':
-                fields = [fname for fname, field in self.group.fields.iteritems()
-                        if field.attrs.get('loading', 'eager') == 'eager']
+                fnames = [fname
+                    for fname, field in self.group.fields.iteritems()
+                    if field.attrs.get('loading', 'eager') == 'eager']
             else:
-                fields = self.group.fields.keys()
-            fields = [fname for fname in fields if fname not in self._loaded]
-            fields.extend(('%s.rec_name' % fname for fname in fields[:]
+                fnames = self.group.fields.keys()
+            fnames = [fname for fname in fnames if fname not in self._loaded]
+            fnames.extend(('%s.rec_name' % fname for fname in fnames[:]
                     if self.group.fields[fname].attrs['type']
                     in ('many2one', 'one2one', 'reference')))
-            fields.append('_timestamp')
-            ctx = rpc.CONTEXT.copy()
-            ctx.update(self.context_get())
-            args = ('model', self.model_name, 'read', ids, fields, ctx)
+            if 'rec_name' not in fnames:
+                fnames.append('rec_name')
+            fnames.append('_timestamp')
+            ctx = record_context.copy()
+            ctx.update(dict(('%s.%s' % (self.model_name, fname), 'size')
+                    for fname, field in self.group.fields.iteritems()
+                    if field.attrs['type'] == 'binary' and fname in fnames))
+            exception = None
             try:
-                values = rpc.execute(*args)
-            except Exception, exception:
-                if raise_exception:
-                    raise
-                values = common.process_exception(exception, self.window, *args)
-                if not values:
-                    values = [{'id': x} for x in ids]
+                values = RPCExecute('model', self.model_name, 'read',
+                    id2record.keys(), fnames, context=ctx,
+                    main_iteration=False)
+            except RPCException, exception:
+                values = [{'id': x} for x in id2record]
+                default_values = dict((f, None) for f in fnames)
+                for value in values:
+                    value.update(default_values)
+                self.exception = True
             id2value = dict((value['id'], value) for value in values)
-            if ids != [self.id]:
-                for id in ids:
-                    record = self.group.get(id)
+            if len(id2record) > 1:
+                for id, record in id2record.iteritems():
+                    if not record.exception:
+                        record.exception = bool(exception)
                     value = id2value.get(id)
-                    if record and value:
+                    if record and not record.destroyed and value:
                         record.set(value, signal=False)
             else:
                 value = id2value.get(self.id)
@@ -118,7 +149,7 @@ class Record(SignalEvent):
         if check_load:
             self._check_load([field_name])
         group = self.value.get(field_name)
-        if group is False or group is None:
+        if group is None:
             return []
 
         if id(group.fields) != id(self.group.fields):
@@ -127,8 +158,6 @@ class Record(SignalEvent):
         group.on_write = self.group.on_write
         group.readonly = self.group.readonly
         group._context.update(self.group._context)
-        if group.window != self.window:
-            group.window = self.window
         return group
 
     def get_path(self, group):
@@ -144,19 +173,6 @@ class Record(SignalEvent):
         path.reverse()
         return tuple(path)
 
-    def _get_window(self):
-        return self.__window
-
-    def _set_window(self, window):
-        self.__window = window
-        for fieldname, value in self.value.iteritems():
-            if fieldname not in self.group.fields:
-                continue
-            if isinstance(self.group.fields[fieldname], field.O2MField):
-                value.window = window
-
-    window = property(_get_window, _set_window)
-
     def get_removed(self):
         if self.group is not None:
             return self in self.group.record_removed
@@ -172,7 +188,7 @@ class Record(SignalEvent):
     deleted = property(get_deleted)
 
     def get_readonly(self):
-        return self.deleted or self.removed
+        return self.deleted or self.removed or self.exception
 
     readonly = property(get_readonly)
 
@@ -227,6 +243,16 @@ class Record(SignalEvent):
         value['id'] = self.id
         return value
 
+    def get_on_change_value(self, check_load=True):
+        if check_load:
+            self._check_load()
+        value = {}
+        for name, field in self.group.fields.iteritems():
+            value[name] = field.get_on_change_value(self,
+                check_load=check_load)
+        value['id'] = self.id
+        return value
+
     def cancel(self):
         self._loaded.clear()
         self.modified_fields.clear()
@@ -241,15 +267,11 @@ class Record(SignalEvent):
         if self.id < 0 or self.modified:
             if self.id < 0:
                 value = self.get(get_readonly=True)
-                args = ('model', self.model_name, 'create', value,
-                    self.context_get())
                 try:
-                    res = rpc.execute(*args)
-                except Exception, exception:
-                    res = common.process_exception(exception, self.window,
-                            *args)
-                    if not res:
-                        return False
+                    res = RPCExecute('model', self.model_name, 'create', value,
+                        context=self.context_get())
+                except RPCException:
+                    return False
                 old_id = self.id
                 self.id = res
                 self.group.id_changed(old_id)
@@ -261,16 +283,12 @@ class Record(SignalEvent):
                     context = self.context_get()
                     context = context.copy()
                     context['_timestamp'] = self.get_timestamp()
-                    args = ('model', self.model_name, 'write', [self.id],
-                            value, context)
                     try:
-                        if not rpc.execute(*args):
-                            return False
-                    except Exception, exception:
-                        res = common.process_exception(exception, self.window,
-                                *args)
-                        if not res:
+                        if not RPCExecute('model', self.model_name, 'write',
+                                [self.id], value, context=context):
                             return False
+                    except RPCException:
+                        return False
             self._loaded.clear()
             self.modified_fields = {}
             if force_reload:
@@ -282,17 +300,44 @@ class Record(SignalEvent):
             self.parent.save(force_reload=force_reload)
         return self.id
 
+    @staticmethod
+    def delete(records, context=None):
+        if not records:
+            return
+        record = records[0]
+        root_group = record.group.root_group
+        assert all(r.model_name == record.model_name for r in records)
+        assert all(r.group.root_group == root_group for r in records)
+        records = [r for r in records if r.id >= 0]
+        ctx = {}
+        ctx.update(context or {})
+        ctx['_timestamp'] = {}
+        for rec in records:
+            ctx['_timestamp'].update(rec.get_timestamp())
+        record_ids = set(r.id for r in records)
+        reload_ids = set(root_group.on_write_ids(list(record_ids)))
+        reload_ids -= record_ids
+        reload_ids = list(reload_ids)
+        try:
+            RPCExecute('model', record.model_name, 'delete', list(record_ids),
+                context=ctx)
+        except RPCException:
+            return False
+        if reload_ids:
+            root_group.reload(reload_ids)
+        return True
+
     def default_get(self, domain=None, context=None):
         if len(self.group.fields):
-            args = ('model', self.model_name, 'default_get',
-                    self.group.fields.keys(), context)
             try:
-                vals = rpc.execute(*args)
-            except Exception, exception:
-                vals = common.process_exception(exception, self.window, *args)
-                if not vals:
-                    return
-            if self.parent_name in self.group.fields and self.parent:
+                vals = RPCExecute('model', self.model_name, 'default_get',
+                    self.group.fields.keys(), context=context)
+            except RPCException:
+                return
+            if (self.parent
+                    and self.parent_name in self.group.fields
+                    and (self.group.fields[self.parent_name].attrs['relation']
+                        == self.group.parent.model_name)):
                 vals[self.parent_name] = self.parent.id
             self.set_default(vals)
         for fieldname, fieldinfo in self.group.fields.iteritems():
@@ -301,16 +346,11 @@ class Record(SignalEvent):
             self.do_autocomplete(fieldname)
 
     def rec_name(self):
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(self.context_get())
-        args = ('model', self.model_name, 'read', self.id, ['rec_name'], ctx)
         try:
-            res = rpc.execute(*args)
-        except Exception, exception:
-            res = common.process_exception(exception, self.window, *args)
-            if not res:
-                return ''
-        return res['rec_name']
+            return RPCExecute('model', self.model_name, 'read', self.id,
+                ['rec_name'], context=self.context_get())['rec_name']
+        except RPCException:
+            return ''
 
     def validate(self, fields=None, softvalidation=False):
         if isinstance(fields, list) and fields:
@@ -321,6 +361,10 @@ class Record(SignalEvent):
         for field_name, field in self.group.fields.iteritems():
             if fields and field_name not in fields:
                 continue
+            if field.get_state_attrs(self).get('readonly', False):
+                continue
+            if field_name == self.group.exclude_field:
+                continue
             if not field.validate(self, softvalidation):
                 res = False
         return res
@@ -337,27 +381,17 @@ class Record(SignalEvent):
     def context_get(self):
         return self.group.context
 
-    def get_default(self):
-        self._check_load()
-        value = dict([(name, field.get_default(self))
-                      for name, field in self.group.fields.iteritems()])
-        return value
-
     def set_default(self, val, signal=True, modified=False):
         for fieldname, value in val.items():
             if fieldname not in self.group.fields:
                 continue
-            if isinstance(self.group.fields[fieldname], field.M2OField):
-                if fieldname + '.rec_name' in val:
-                    value = (value, val[fieldname + '.rec_name'])
-            elif isinstance(self.group.fields[fieldname], field.ReferenceField):
-                if value:
-                    ref_model, ref_id = value.split(',', 1)
-                    if fieldname + '.rec_name' in val:
-                        value = ref_model, (ref_id,
-                            val[fieldname + '.rec_name'])
-                    else:
-                        value = ref_model, (ref_id, ref_id)
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in val:
+                    self.value[field_rec_name] = val[field_rec_name]
+                elif field_rec_name in self.value:
+                    del self.value[field_rec_name]
             self.group.fields[fieldname].set_default(self, value,
                 modified=modified)
             self._loaded.add(fieldname)
@@ -372,21 +406,19 @@ class Record(SignalEvent):
                 self._timestamp = value
                 continue
             if fieldname not in self.group.fields:
+                if fieldname == 'rec_name':
+                    self.value['rec_name'] = value
                 continue
-            if isinstance(self.group.fields[fieldname], field.O2MField):
+            if isinstance(self.group.fields[fieldname], fields.O2MField):
                 later[fieldname] = value
                 continue
-            if isinstance(self.group.fields[fieldname], field.M2OField):
-                if fieldname + '.rec_name' in val:
-                    value = (value, val[fieldname + '.rec_name'])
-            elif isinstance(self.group.fields[fieldname], field.ReferenceField):
-                if value:
-                    ref_model, ref_id = value.split(',', 1)
-                    if fieldname + '.rec_name' in val:
-                        value = ref_model, (ref_id,
-                            val[fieldname + '.rec_name'])
-                    else:
-                        value = ref_model, (ref_id, ref_id)
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in val:
+                    self.value[field_rec_name] = val[field_rec_name]
+                elif field_rec_name in self.value:
+                    del self.value[field_rec_name]
             self.group.fields[fieldname].set(self, value, modified=False)
             self._loaded.add(fieldname)
         for fieldname, value in later.iteritems():
@@ -395,7 +427,6 @@ class Record(SignalEvent):
         if modified:
             self.modified_fields.update(dict((x, None) for x in val))
             self.signal('record-modified')
-            self.signal('record-changed')
         if signal:
             self.signal('record-changed')
 
@@ -415,11 +446,13 @@ class Record(SignalEvent):
         if check_load:
             self._check_load()
         ctx = rpc.CONTEXT.copy()
+        ctx['context'] = ctx.copy()
         for name, field in self.group.fields.items():
             ctx[name] = field.get_eval(self, check_load=check_load)
 
-        ctx['context'] = self.context_get()
+        ctx['context'].update(self.context_get())
         ctx['active_id'] = self.id
+        ctx['id'] = self.id  # Force local id
         ctx['_user'] = rpc._USER
         if self.parent and self.parent_name:
             ctx['_parent_' + self.parent_name] = \
@@ -429,12 +462,7 @@ class Record(SignalEvent):
 
     def _get_on_change_args(self, args):
         res = {}
-        values = {}
-        for name, field in self.group.fields.iteritems():
-            values[name] = field.get_on_change_value(self, check_load=False)
-        if self.parent and self.parent_name:
-            values['_parent_' + self.parent_name] = \
-                    common.EvalEnvironment(self.parent, False, 'on_change')
+        values = common.EvalEnvironment(self, True, 'on_change')
         for arg in args:
             scope = values
             for i in arg.split('.'):
@@ -449,34 +477,25 @@ class Record(SignalEvent):
         if isinstance(attr, basestring):
             attr = PYSONDecoder().decode(attr)
         args = self._get_on_change_args(attr)
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(self.context_get())
-        args = ('model', self.model_name, 'on_change_' + fieldname, args, ctx)
         try:
-            res = rpc.execute(*args)
-        except Exception, exception:
-            res = common.process_exception(exception, self.window, *args)
-            if not res:
-                return
+            res = RPCExecute('model', self.model_name,
+                'on_change_' + fieldname, args, context=self.context_get())
+        except RPCException:
+            return
         later = {}
         for fieldname, value in res.items():
             if fieldname not in self.group.fields:
                 continue
-            if isinstance(self.group.fields[fieldname], field.O2MField):
+            if isinstance(self.group.fields[fieldname], fields.O2MField):
                 later[fieldname] = value
                 continue
-            if isinstance(self.group.fields[fieldname], field.M2OField):
-                if fieldname + '.rec_name' in res:
-                    value = (value, res[fieldname + '.rec_name'])
-            elif isinstance(self.group.fields[fieldname],
-                    field.ReferenceField):
-                if value:
-                    ref_model, ref_id = value.split(',', 1)
-                    if fieldname + '.rec_name' in res:
-                        value = ref_model, (ref_id,
-                                res[fieldname + '.rec_name'])
-                    else:
-                        value = ref_model, (ref_id, ref_id)
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in res:
+                    self.value[field_rec_name] = res[field_rec_name]
+                elif field_rec_name in self.value:
+                    del self.value[field_rec_name]
             self.group.fields[fieldname].set_on_change(self, value)
         for fieldname, value in later.items():
             # on change recursion checking is done only for x2many
@@ -488,6 +507,9 @@ class Record(SignalEvent):
                 field_x2many.in_on_change = False
 
     def on_change_with(self, field_name):
+        fieldnames = set()
+        values = {}
+        later = set()
         for fieldname in self.group.fields:
             on_change_with = self.group.fields[fieldname].attrs.get(
                     'on_change_with')
@@ -497,18 +519,28 @@ class Record(SignalEvent):
                 continue
             if field_name == fieldname:
                 continue
-            args = self._get_on_change_args(on_change_with)
-            ctx = rpc.CONTEXT.copy()
-            ctx.update(self.context_get())
-            args = ('model', self.model_name, 'on_change_with_' + fieldname,
-                    args, ctx)
+            if fieldnames & set(on_change_with):
+                later.add(fieldname)
+                continue
+            fieldnames.add(fieldname)
+            values.update(self._get_on_change_args(on_change_with))
+        if fieldnames:
             try:
-                res = rpc.execute(*args)
-            except Exception, exception:
-                res = common.process_exception(exception, self.window, *args)
-                if not res:
-                    return
-            self.group.fields[fieldname].set_on_change(self, res)
+                result = RPCExecute('model', self.model_name, 'on_change_with',
+                    list(fieldnames), values, context=self.context_get())
+            except RPCException:
+                return
+            for fieldname, value in result.items():
+                self.group.fields[fieldname].set_on_change(self, value)
+        for fieldname in later:
+            values = self._get_on_change_args(on_change_with)
+            try:
+                result = RPCExecute('model', self.model_name,
+                    'on_change_with_' + fieldname, values,
+                    context=self.context_get())
+            except RPCException:
+                return
+            self.group.fields[fieldname].set_on_change(self, result)
 
     def autocomplete_with(self, field_name):
         for fieldname, fieldinfo in self.group.fields.iteritems():
@@ -521,48 +553,35 @@ class Record(SignalEvent):
         self.autocompletion[fieldname] = []
         autocomplete = self.group.fields[fieldname].attrs['autocomplete']
         args = self._get_on_change_args(autocomplete)
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(self.context_get())
-        args = ('model', self.model_name, 'autocomplete_' + fieldname, args,
-            ctx)
         try:
-            res = rpc.execute(*args)
-        except Exception, exception:
-            res = common.process_exception(exception, self.window, *args)
-            if not res:
-                # ensure res is a list
-                res = []
+            res = RPCExecute('model', self.model_name, 'autocomplete_' +
+                fieldname, args, context=self.context_get())
+        except RPCException:
+            # ensure res is a list
+            res = []
         self.autocompletion[fieldname] = res
 
-    def cond_default(self, field_name, value):
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(self.context_get())
-        args = ('model', 'ir.default', 'get_default', self.model_name,
-                field_name + '=' + str(value), ctx)
-        try:
-            res = rpc.execute(*args)
-        except Exception, exception:
-            res = common.process_exception(exception, self.window, *args)
-            if not res:
-                return
-        self.set_default(res)
-
     def get_attachment_count(self, reload=False):
         if self.id < 0:
             return 0
         if self.attachment_count < 0 or reload:
-            args = ('model', 'ir.attachment', 'search_count', [
-                ('resource', '=', '%s,%s' % (self.model_name, self.id)),
-                ], rpc.CONTEXT)
             try:
-                self.attachment_count = rpc.execute(*args)
-            except Exception:
+                self.attachment_count = RPCExecute('model', 'ir.attachment',
+                    'search_count', [
+                        ('resource', '=',
+                            '%s,%s' % (self.model_name, self.id)),
+                        ])
+            except RPCException:
                 return 0
         return self.attachment_count
 
     def destroy(self):
+        for v in self.value.itervalues():
+            if hasattr(v, 'destroy'):
+                v.destroy()
         super(Record, self).destroy()
-        self.window = None
         self.group = None
         self.value = None
         self.next = None
+        self.destroyed = True
+        POOL[self.model_name].remove(self)
