@@ -19,6 +19,7 @@ class FloatTime(WidgetInterface):
         self.entry.connect('activate', self.sig_activate)
         self.entry.connect('focus-in-event', lambda x, y: self._focus_in())
         self.entry.connect('focus-out-event', lambda x, y: self._focus_out())
+        self.entry.connect('key-press-event', self.send_modified)
         self.widget.pack_start(self.entry)
 
         self.conv = None
@@ -31,17 +32,24 @@ class FloatTime(WidgetInterface):
     def grab_focus(self):
         return self.entry.grab_focus()
 
+    @property
+    def modified(self):
+        if self.record and self.field:
+            value = self.entry.get_text()
+            return common.float_time_to_text(self.field.get(self.record),
+                self.conv) != value
+        return False
+
     def set_value(self, record, field):
         value = self.entry.get_text()
-        if not value:
-            return field.set_client(record, 0.0)
+        digits = field.digits(record)
         return field.set_client(record,
-                common.text_to_float_time(value, self.conv))
+            common.text_to_float_time(value, self.conv, digits[1]))
 
     def display(self, record, field):
         super(FloatTime, self).display(record, field)
         if not field:
-            self.entry.set_text('00:00')
+            self.entry.set_text('')
             return False
         val = field.get(record)
 
