@@ -2,16 +2,13 @@
 #this repository contains the full copyright notices and license terms.
 import gtk
 import gettext
-from tryton.common import TRYTON_ICON, COLOR_SCHEMES
 from interface import WidgetInterface
 from tryton.gui.window.view_form.screen import Screen
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
 from tryton.config import CONFIG
-from tryton.exceptions import TrytonServerError
 import tryton.common as common
-import tryton.rpc as rpc
-import pango
+from tryton.common import RPCExecute, RPCException
 
 _ = gettext.gettext
 
@@ -47,17 +44,19 @@ class One2Many(WidgetInterface):
             tooltips.set_tip(self.but_add, _('Add'))
             self.but_add.connect('clicked', self._sig_add)
             img_add = gtk.Image()
-            img_add.set_from_stock('tryton-list-add', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_add.set_from_stock('tryton-list-add',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_add.set_alignment(0.5, 0.5)
             self.but_add.add(img_add)
             self.but_add.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_add, expand=False, fill=False)
 
             self.but_remove = gtk.Button()
-            tooltips.set_tip(self.but_remove, _('Remove'))
+            tooltips.set_tip(self.but_remove, _('Remove <Del>'))
             self.but_remove.connect('clicked', self._sig_remove, True)
             img_remove = gtk.Image()
-            img_remove.set_from_stock('tryton-list-remove', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_remove.set_from_stock('tryton-list-remove',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_remove.set_alignment(0.5, 0.5)
             self.but_remove.add(img_remove)
             self.but_remove.set_relief(gtk.RELIEF_NONE)
@@ -66,7 +65,7 @@ class One2Many(WidgetInterface):
             hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
         self.but_new = gtk.Button()
-        tooltips.set_tip(self.but_new, _('Create a new record'))
+        tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
         self.but_new.connect('clicked', self._sig_new)
         img_new = gtk.Image()
         img_new.set_from_stock('tryton-new', gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -76,7 +75,7 @@ class One2Many(WidgetInterface):
         hbox.pack_start(self.but_new, expand=False, fill=False)
 
         self.but_open = gtk.Button()
-        tooltips.set_tip(self.but_open, _('Edit selected record'))
+        tooltips.set_tip(self.but_open, _('Edit selected record <F2>'))
         self.but_open.connect('clicked', self._sig_edit)
         img_open = gtk.Image()
         img_open.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -86,7 +85,7 @@ class One2Many(WidgetInterface):
         hbox.pack_start(self.but_open, expand=False, fill=False)
 
         self.but_del = gtk.Button()
-        tooltips.set_tip(self.but_del, _('Delete selected record'))
+        tooltips.set_tip(self.but_del, _('Delete selected record <Del>'))
         self.but_del.connect('clicked', self._sig_remove, False)
         img_del = gtk.Image()
         img_del.set_from_stock('tryton-delete', gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -96,7 +95,7 @@ class One2Many(WidgetInterface):
         hbox.pack_start(self.but_del, expand=False, fill=False)
 
         self.but_undel = gtk.Button()
-        tooltips.set_tip(self.but_undel, _('Undelete selected record'))
+        tooltips.set_tip(self.but_undel, _('Undelete selected record <Ins>'))
         self.but_undel.connect('clicked', self._sig_undelete)
         img_undel = gtk.Image()
         img_undel.set_from_stock('tryton-undo', gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -111,7 +110,8 @@ class One2Many(WidgetInterface):
         tooltips.set_tip(self.but_pre, _('Previous'))
         self.but_pre.connect('clicked', self._sig_previous)
         img_pre = gtk.Image()
-        img_pre.set_from_stock('tryton-go-previous', gtk.ICON_SIZE_SMALL_TOOLBAR)
+        img_pre.set_from_stock('tryton-go-previous',
+            gtk.ICON_SIZE_SMALL_TOOLBAR)
         img_pre.set_alignment(0.5, 0.5)
         self.but_pre.add(img_pre)
         self.but_pre.set_relief(gtk.RELIEF_NONE)
@@ -136,7 +136,8 @@ class One2Many(WidgetInterface):
         tooltips.set_tip(but_switch, _('Switch'))
         but_switch.connect('clicked', self.switch_view)
         img_switch = gtk.Image()
-        img_switch.set_from_stock('tryton-fullscreen', gtk.ICON_SIZE_SMALL_TOOLBAR)
+        img_switch.set_from_stock('tryton-fullscreen',
+            gtk.ICON_SIZE_SMALL_TOOLBAR)
         img_switch.set_alignment(0.5, 0.5)
         but_switch.add(img_switch)
         but_switch.set_relief(gtk.RELIEF_NONE)
@@ -156,6 +157,7 @@ class One2Many(WidgetInterface):
 
         self.screen = Screen(attrs['relation'],
             mode=attrs.get('mode', 'tree,form').split(','),
+            view_ids=attrs.get('view_ids', '').split(','),
             views_preload=attrs.get('views', {}),
             row_activate=self._on_activate,
             exclude_field=attrs.get('relation_field', None))
@@ -202,6 +204,10 @@ class One2Many(WidgetInterface):
         self.screen.switch_view()
         self.color_set(self.color_name)
 
+    @property
+    def modified(self):
+        return self.screen.current_view.modified
+
     def color_set(self, name):
         super(One2Many, self).color_set(name)
         widget = self._color_widget()
@@ -230,7 +236,7 @@ class One2Many(WidgetInterface):
                 self.screen.display()
                 return
         ctx = {}
-        ctx.update(self.record.expr_eval(self.attrs.get('context', {})))
+        ctx.update(self.field.context_get(self.record))
         sequence = None
         if self.screen.current_view.view_type == 'tree':
             sequence = self.screen.current_view.widget_tree.sequence
@@ -295,40 +301,37 @@ class One2Many(WidgetInterface):
     def _sig_add(self, *args):
         self.view.set_value()
         domain = self.field.domain_get(self.record)
-        context = rpc.CONTEXT.copy()
-        context.update(self.field.context_get(self.record))
+        context = self.field.context_get(self.record)
         domain = domain[:]
-        domain.extend(self.record.expr_eval(self.attrs.get('add_remove'),
-            context))
+        domain.extend(self.record.expr_eval(self.attrs.get('add_remove')))
         removed_ids = self.field.get_removed_ids(self.record)
 
         try:
             if self.wid_text.get_text():
-                dom = [('rec_name', 'ilike', '%' + self.wid_text.get_text() + '%'),
+                dom = [('rec_name', 'ilike',
+                        '%' + self.wid_text.get_text() + '%'),
                     ['OR', domain, ('id', 'in', removed_ids)]]
             else:
                 dom = ['OR', domain, ('id', 'in', removed_ids)]
-            ids = rpc.execute('model', self.attrs['relation'], 'search', dom,
-                    0, CONFIG['client.limit'], None, context)
-        except TrytonServerError, exception:
-            common.process_exception(exception)
+            ids = RPCExecute('model', self.attrs['relation'], 'search', dom,
+                    0, CONFIG['client.limit'], None, context=context)
+        except RPCException:
             return False
-        def callback(ids):
-            res_id = None
-            if ids:
-                res_id = ids[0]
-            self.screen.load(ids, modified=True)
-            self.screen.display(res_id=res_id)
-            if self.screen.current_view:
-                self.screen.current_view.set_cursor()
+
+        def callback(result):
+            if result:
+                ids = [x[0] for x in result]
+                self.screen.load(ids, modified=True)
+                self.screen.display(res_id=ids[0])
+            self.screen.set_cursor()
             self.wid_text.set_text('')
         if len(ids) != 1:
             WinSearch(self.attrs['relation'], callback, sel_multi=True,
                 ids=ids, context=context, domain=domain,
+                view_ids=self.attrs.get('view_ids', '').split(','),
                 views_preload=self.attrs.get('views', {}))
         else:
-            callback(ids)
-
+            callback([(i, None) for i in ids])
 
     def _sig_label(self, screen, signal_data):
         name = '_'
@@ -387,7 +390,7 @@ class One2Many(WidgetInterface):
     def set_value(self, record, field):
         self.screen.save_tree_state()
         self.screen.current_view.set_value()
-        if self.screen.modified(): # TODO check if required
+        if self.screen.modified():  # TODO check if required
             record.modified_fields.setdefault(field.name)
             record.signal('record-modified')
         return True
