@@ -5,11 +5,10 @@ from interface import ParserView
 
 class ViewGraph(ParserView):
 
-    def __init__(self, window, screen, widget, children=None,
-            buttons=None, toolbar=None, notebooks=None, cursor_widget=None,
-            children_field=None):
-        super(ViewGraph, self).__init__(window, screen, widget, children,
-                buttons, toolbar, notebooks, cursor_widget, children_field)
+    def __init__(self, screen, widget, children=None, buttons=None,
+            notebooks=None, cursor_widget=None, children_field=None):
+        super(ViewGraph, self).__init__(screen, widget, children, buttons,
+            notebooks, cursor_widget, children_field)
         self.view_type = 'graph'
         self.widgets = children
 
@@ -21,9 +20,9 @@ class ViewGraph(ParserView):
         for widget in self.widgets.keys():
             self.widgets[widget].destroy()
             del self.widgets[widget]
-        del self.widget
-        del self.screen
-        del self.buttons
+        self.widget = None
+        self.screen = None
+        self.buttons = None
 
     def cancel(self):
         pass
