@@ -1,8 +1,9 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-#This code is inspired by the pycha project (http://www.lorenzogil.com/projects/pycha/)
+#This code is inspired by the pycha project
+#(http://www.lorenzogil.com/projects/pycha/)
 from graph import Graph
-from tryton.common import hex2rgb, lighten, float_time_to_text
+from tryton.common import float_time_to_text
 import locale
 import math
 import cairo
@@ -21,8 +22,8 @@ class Bar(Graph):
             w = self.area.w * bar.w
             h = self.area.h * bar.h
 
-            if w < 1 or h <1:
-                return # don't draw too small
+            if w < 1 or h < 1:
+                return  # don't draw too small
 
             cr.set_source_rgba(0, 0, 0, 0.15)
             rectangle = self._getShadowRectangle(x, y, w, h)
@@ -52,7 +53,8 @@ class Bar(Graph):
 
     def sourceRectangle(self, x, y, w, h, r, g, b):
         linear = cairo.LinearGradient((x + w) / 2, y, (x + w) / 2, y + h)
-        linear.add_color_stop_rgb(0, 3.5 * r / 5.0, 3.5 * g / 5.0, 3.5 * b / 5.0)
+        linear.add_color_stop_rgb(0, 3.5 * r / 5.0, 3.5 * g / 5.0,
+            3.5 * b / 5.0)
         linear.add_color_stop_rgb(1, r, g, b)
         return linear
 
@@ -71,8 +73,9 @@ class Bar(Graph):
 
         highlight = False
         draw_bars = []
-        yfields_float_time = dict([(x.get('key', x['name']), x.get('float_time'))
-                for x in self.yfields if x.get('widget')])
+        yfields_float_time = dict(
+            (x.get('key', x['name']), x.get('float_time'))
+            for x in self.yfields if x.get('widget'))
         for bar in self.bars:
             if intersect(bar, event):
                 if not bar.highlight:
@@ -179,7 +182,7 @@ class VerticalBar(Bar):
         return ylabels
 
     def _getShadowRectangle(self, x, y, w, h):
-        return (x-2, y-2, w+4, h+2)
+        return (x - 2, y - 2, w + 4, h + 2)
 
 
 class HorizontalBar(Bar):
@@ -236,7 +239,7 @@ class HorizontalBar(Bar):
         return [(x[0], x[1]) for x in ylabels]
 
     def _getShadowRectangle(self, x, y, w, h):
-        return (x, y-2, w+2, h+4)
+        return (x, y - 2, w + 2, h + 4)
 
     def _getLegendPosition(self, width, height):
         return self.area.x + self.area.w * 0.95 - width, \
