@@ -5,9 +5,8 @@
 
 class ParserInterface(object):
 
-    def __init__(self, window, parent=None, attrs=None, screen=None,
+    def __init__(self, parent=None, attrs=None, screen=None,
             children_field=None):
-        self.window = window
         self.parent = parent
         self.attrs = attrs
         self.title = None
@@ -18,17 +17,14 @@ class ParserInterface(object):
 
 class ParserView(object):
 
-    def __init__(self, window, screen, widget, children=None, buttons=None,
-            toolbar=None, notebooks=None, cursor_widget=None,
-            children_field=None):
-        self.window = window
+    def __init__(self, screen, widget, children=None, buttons=None,
+            notebooks=None, cursor_widget=None, children_field=None):
         self.screen = screen
         self.widget = widget
         self.children = children
         if buttons is None:
             buttons = []
         self.buttons = buttons
-        self.toolbar = toolbar
         if notebooks is None:
             notebooks = []
         self.notebooks = notebooks
@@ -37,3 +33,7 @@ class ParserView(object):
 
     def get_fields(self):
         return self.children.keys()
+
+    @property
+    def modified(self):
+        return False
