@@ -3,26 +3,42 @@
 import gobject
 import gtk
 import sys
-if sys.version_info < (2, 6):
+try:
     import simplejson as json
-else:
+except ImportError:
     import json
-import tryton.rpc as rpc
 import locale
 from interface import ParserView
 from tryton.action import Action
 from tryton.common import message
 import gettext
-import tryton.common as common
 from tryton.config import CONFIG
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrenderertoggle import CellRendererToggle
+from tryton.common.cellrendererbinary import CellRendererBinary
 from tryton.pyson import PYSONEncoder
-import os
+from tryton.gui.window import Window
+from tryton.common.popup_menu import populate
+from tryton.common import RPCExecute, RPCException
 
 _ = gettext.gettext
 
 
+def path_convert_id2pos(model, id_path):
+    "This function will transform a path of id into a path of position"
+    group = model.group
+    id_path = id_path[:]
+    while id_path:
+        current_id = id_path.pop(0)
+        try:
+            record = group.get(current_id)
+            group = record.children_group(model.children_field,
+                check_load=True)
+        except (KeyError, AttributeError):
+            return None
+    return model.on_get_path(record)
+
+
 class AdaptModelGroup(gtk.GenericTreeModel):
 
     def __init__(self, group, children_field=None):
@@ -32,7 +48,7 @@ class AdaptModelGroup(gtk.GenericTreeModel):
         self.sort_asc = True
         self.set_property('leak_references', False)
         self.children_field = children_field
-        self.__removed = None # XXX dirty hack to allow update of has_child
+        self.__removed = None  # XXX dirty hack to allow update of has_child
 
     def added(self, group, record):
         if (group is self.group
@@ -63,7 +79,7 @@ class AdaptModelGroup(gtk.GenericTreeModel):
                     len(record.children_group(self.children_field)) <= 1):
                 path = self.on_get_path(record.parent)
                 iter_ = self.get_iter(path)
-                self.__removed = record # XXX check for thread
+                self.__removed = record  # XXX check for thread
                 self.row_has_child_toggled(path, iter_)
                 self.__removed = None
 
@@ -114,14 +130,6 @@ class AdaptModelGroup(gtk.GenericTreeModel):
             record.modified_fields.setdefault(record.parent_name or 'id')
         group.move(record, 0)
 
-    def move_end(self, record):
-        group = self.group
-        if group != record.group:
-            record.group.remove(record, remove=True)
-            group.add(record)
-            record.modified_fields.setdefault(record.parent_name or 'id')
-        group.move(record, -1)
-
     def sort(self, ids):
         ids2pos = {}
         pos = 0
@@ -137,7 +145,7 @@ class AdaptModelGroup(gtk.GenericTreeModel):
                 if old_pos != pos:
                     new_order[old_pos] = pos
                 pos += 1
-            except Exception:
+            except KeyError:
                 continue
         self.group.sort(lambda x, y: \
                 cmp(new_order[ids2pos[x.id]], new_order[ids2pos[y.id]]))
@@ -199,12 +207,12 @@ class AdaptModelGroup(gtk.GenericTreeModel):
         return record
 
     def on_iter_next(self, record):
-        if not record:
+        if record is None:
             return None
         return record.next.get(id(record.group))
 
     def on_iter_has_child(self, record):
-        if not self.children_field:
+        if record is None or not self.children_field:
             return False
         children = record.children_group(self.children_field)
         length = len(children)
@@ -213,8 +221,12 @@ class AdaptModelGroup(gtk.GenericTreeModel):
         return bool(length)
 
     def on_iter_children(self, record):
-        if self.children_field and record.children_group(self.children_field):
-            return record.children_group(self.children_field)[0]
+        if record is None:
+            return None
+        if self.children_field:
+            children = record.children_group(self.children_field)
+            if children:
+                return children[0]
         return None
 
     def on_iter_n_children(self, record):
@@ -239,11 +251,10 @@ class AdaptModelGroup(gtk.GenericTreeModel):
 
 class ViewList(ParserView):
 
-    def __init__(self, window, screen, widget, children=None, buttons=None,
-            toolbar=None, notebooks=None, cursor_widget=None,
-            children_field=None):
-        super(ViewList, self).__init__(window, screen, widget, children,
-                buttons, toolbar, notebooks, cursor_widget, children_field)
+    def __init__(self, screen, widget, children=None, buttons=None,
+            notebooks=None, cursor_widget=None, children_field=None):
+        super(ViewList, self).__init__(screen, widget, children, buttons,
+            notebooks, cursor_widget, children_field)
         self.store = None
         self.view_type = 'tree'
 
@@ -257,9 +268,6 @@ class ViewList(ParserView):
         viewport.add(scroll)
         self.widget_tree = self.widget
 
-        if isinstance(self.screen.window, gtk.Dialog):
-            width, height = self.widget_tree.size_request()
-            vbox.set_size_request(width or -1, height or -1)
         vbox.pack_start(viewport, expand=True, fill=True)
 
         self.widget_tree.screen = screen
@@ -280,62 +288,6 @@ class ViewList(ParserView):
                 hbox.pack_start(hbox2, expand=False, fill=False, padding=12)
             hbox.show_all()
 
-        if toolbar and not CONFIG['client.modepda'] \
-                and (toolbar['print'] or toolbar['action']):
-            hbox = gtk.HBox()
-            self.widget.pack_start(hbox, expand=False, fill=False)
-
-            gtktoolbar = gtk.Toolbar()
-            gtktoolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
-            gtktoolbar.set_style(gtk.TOOLBAR_BOTH)
-            hbox.pack_start(gtktoolbar, True, True)
-
-            for icontype in ('print', 'action'):
-                if not toolbar[icontype]:
-                    continue
-
-                for tool in toolbar[icontype]:
-                    if not tool['icon.rec_name']:
-                        iconstock = {
-                            'print': 'tryton-print',
-                            'action': 'tryton-executable',
-                        }.get(icontype)
-                    else:
-                        iconstock = tool['icon.rec_name']
-                    common.ICONFACTORY.register_icon(iconstock)
-
-                    if hasattr(gtk, 'MenuToolButton') and icontype == 'print':
-                        tbutton = gtk.MenuToolButton(iconstock)
-                    else:
-                        tbutton = gtk.ToolButton(iconstock)
-                    tbutton.set_use_underline(True)
-                    text = tool['name']
-                    if '_' not in text:
-                        text = '_' + text
-                    tbutton.set_label(text)
-                    gtktoolbar.insert(tbutton, -1)
-
-                    tbutton.connect('clicked', self._sig_clicked, tool,
-                            icontype)
-                    if hasattr(gtk, 'MenuToolButton') and icontype == 'print':
-                        menu = gtk.Menu()
-                        for mtype, text in (('print', _('_Direct Print')),
-                                ('email', _('_Email as Attachment'))):
-                            menuitem = gtk.MenuItem(text)
-                            tool2 = tool.copy()
-                            if mtype == 'print':
-                                tool2['direct_print'] = True
-                                tool2['email_print'] = False
-                            else:
-                                tool2['direct_print'] = False
-                                tool2['email_print'] = True
-                            menuitem.connect('activate', self._sig_clicked,
-                                    tool2, icontype)
-                            menu.add(menuitem)
-                            menuitem.show()
-                        tbutton.set_menu(menu)
-            hbox.show_all()
-
         self.display()
 
         self.widget_tree.connect('button-press-event', self.__button_press)
@@ -355,64 +307,43 @@ class ViewList(ParserView):
         elif self.widget_tree.sequence:
             dnd = True
         # Disable DnD on mac until it is fully supported
-        if os.name == 'mac':
-            dnd = False
-        elif hasattr(os, 'uname') and os.uname()[0] == 'Darwin':
+        if sys.platform == 'darwin':
             dnd = False
         if screen.readonly:
             dnd = False
         if dnd:
-            self.widget_tree.drag_source_set(gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
-                    [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0),],
-                    gtk.gdk.ACTION_MOVE)
+            self.widget_tree.drag_source_set(
+                gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
+                [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0)],
+                gtk.gdk.ACTION_MOVE)
             self.widget_tree.drag_dest_set(gtk.DEST_DEFAULT_ALL,
-                    [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0),],
-                    gtk.gdk.ACTION_MOVE)
+                [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0)],
+                gtk.gdk.ACTION_MOVE)
 
             self.widget_tree.connect('drag-begin', self.drag_begin)
             self.widget_tree.connect('drag-motion', self.drag_motion)
             self.widget_tree.connect('drag-drop', self.drag_drop)
             self.widget_tree.connect("drag-data-get", self.drag_data_get)
-            self.widget_tree.connect('drag-data-received', self.drag_data_received)
+            self.widget_tree.connect('drag-data-received',
+                self.drag_data_received)
             self.widget_tree.connect('drag-data-delete', self.drag_data_delete)
 
         self.widget_tree.connect('key_press_event', self.on_keypress)
-        self.widget_tree.connect('test-expand-row', self.test_expand_row)
-        self.widget_tree.set_expander_column(self.widget_tree.get_column(0))
+        if self.children_field:
+            self.widget_tree.connect('test-expand-row', self.test_expand_row)
+            self.widget_tree.set_expander_column(
+                self.widget_tree.get_column(0))
+
+    @property
+    def modified(self):
+        return False
 
     def get_fields(self):
         return [col.name for col in self.widget_tree.get_columns() if col.name]
 
-    def _sig_clicked(self, widget, action, atype):
-        return self._action(action, atype)
-
-    def _action(self, action, atype):
-        act = action.copy()
-        obj_ids = self.screen.sel_ids_get()
-        obj_id = self.screen.id_get()
-        if not obj_ids or not obj_id:
-            message(_('No record selected!'), self.window)
-            return False
-        email = {}
-        if action.get('pyson_email'):
-            email = self.screen.current_record.expr_eval(action['pyson_email'])
-            if not email:
-                email = {}
-        email['subject'] = action['name'].replace('_', '')
-        act['email'] = email
-        data = {
-            'model': self.screen.model_name,
-            'id': obj_id,
-            'ids': obj_ids,
-        }
-        value = Action._exec_action(act, self.window, data, {})
-        if self.screen:
-            self.screen.reload(written=True)
-        return value
-
-
     def on_keypress(self, widget, event):
-        if event.keyval == gtk.keysyms.c and event.state & gtk.gdk.CONTROL_MASK:
+        if (event.keyval == gtk.keysyms.c
+                and event.state & gtk.gdk.CONTROL_MASK):
             self.on_copy()
             return False
         if event.keyval in (gtk.keysyms.Down, gtk.keysyms.Up):
@@ -423,7 +354,7 @@ class ViewList(ParserView):
             if event.keyval == gtk.keysyms.Down:
                 test = True
                 for i in xrange(len(path)):
-                    iter_ = model.get_iter(path[0:i+1])
+                    iter_ = model.get_iter(path[0:i + 1])
                     if model.iter_next(iter_):
                         test = False
                 if test:
@@ -435,7 +366,8 @@ class ViewList(ParserView):
             elif event.keyval == gtk.keysyms.Up:
                 if path == (0,):
                     return True
-        if event.keyval in (gtk.keysyms.Left, gtk.keysyms.Right):
+        if (event.keyval in (gtk.keysyms.Left, gtk.keysyms.Right)
+                and self.children_field):
             selection = widget.get_selection()
             model, paths = selection.get_selected_rows()
             if event.keyval == gtk.keysyms.Left:
@@ -462,38 +394,38 @@ class ViewList(ParserView):
         while iter_:
             record = model.get_value(iter_, 0)
             if not record.get_loaded(fields):
-                try:
-                    for field in fields:
-                        record.__getitem__(field, True)
-                except Exception, exception:
-                    return True
+                for field in fields:
+                    record[field]
+                    if record.exception:
+                        return True
             iter_ = model.iter_next(iter_)
         return False
 
     def on_copy(self):
-        clipboard = self.widget_tree.get_clipboard(gtk.gdk.SELECTION_CLIPBOARD)
-        targets = [
-            ('STRING', 0, 0),
-            ('TEXT', 0, 1),
-            ('COMPOUND_TEXT', 0, 2),
-            ('UTF8_STRING', 0, 3)
-        ]
-        selection = self.widget_tree.get_selection()
-        # Set to clipboard directly if not too much selected rows
-        # to speed up paste
-        # Don't use set_with_data on mac see:
-        # http://bugzilla.gnome.org/show_bug.cgi?id=508601
-        if selection.count_selected_rows() < 100 \
-                or os.name == 'mac' \
-                or (hasattr(os, 'uname') and os.uname()[0] == 'Darwin'):
-            data = []
-            selection.selected_foreach(self.copy_foreach, data)
-            clipboard.set_text('\n'.join(data))
-        else:
-            clipboard.set_with_data(targets, self.copy_get_func,
-                    self.copy_clear_func, selection)
+        for clipboard_type in (gtk.gdk.SELECTION_CLIPBOARD,
+                gtk.gdk.SELECTION_PRIMARY):
+            clipboard = self.widget_tree.get_clipboard(clipboard_type)
+            targets = [
+                ('STRING', 0, 0),
+                ('TEXT', 0, 1),
+                ('COMPOUND_TEXT', 0, 2),
+                ('UTF8_STRING', 0, 3)
+            ]
+            selection = self.widget_tree.get_selection()
+            # Set to clipboard directly if not too much selected rows
+            # to speed up paste
+            # Don't use set_with_data on mac see:
+            # http://bugzilla.gnome.org/show_bug.cgi?id=508601
+            if selection.count_selected_rows() < 100 \
+                    or sys.platform == 'darwin':
+                data = []
+                selection.selected_foreach(self.copy_foreach, data)
+                clipboard.set_text('\n'.join(data))
+            else:
+                clipboard.set_with_data(targets, self.copy_get_func,
+                        self.copy_clear_func, selection)
 
-    def copy_foreach(self, treemodel, path,iter, data):
+    def copy_foreach(self, treemodel, path, iter, data):
         record = treemodel.get_value(iter, 0)
         values = []
         for col in self.widget_tree.get_columns():
@@ -539,6 +471,7 @@ class ViewList(ParserView):
     def drag_data_get(self, treeview, context, selection, target_id,
             etime):
         treeview.emit_stop_by_name('drag-data-get')
+
         def _func_sel_get(store, path, iter_, data):
             value = store.get_value(iter_, 0)
             data.append(json.dumps(value.get_path(store.group)))
@@ -570,6 +503,7 @@ class ViewList(ParserView):
         record = store.group.get_by_path(data)
         record_path = store.on_get_path(record)
         drop_info = treeview.get_dest_row_at_pos(x, y)
+
         def check_recursion(from_, to):
             if not from_ or not to:
                 return True
@@ -582,7 +516,8 @@ class ViewList(ParserView):
         if drop_info:
             path, position = drop_info
             check_path = path
-            if position in (gtk.TREE_VIEW_DROP_BEFORE, gtk.TREE_VIEW_DROP_AFTER):
+            if position in (gtk.TREE_VIEW_DROP_BEFORE,
+                    gtk.TREE_VIEW_DROP_AFTER):
                 check_path = path[:-1]
             if not check_recursion(record_path, check_path):
                 return
@@ -593,7 +528,7 @@ class ViewList(ParserView):
             elif self.children_field:
                 store.move_into(record, path)
         else:
-            store.move_end(record)
+            store.move_after(record, (len(store) - 1,))
         context.drop_finish(False, etime)
         if treeview.sequence:
             record.group.set_sequence(field=treeview.sequence)
@@ -612,47 +547,45 @@ class ViewList(ParserView):
                 model = selection.get_selected_rows()[0]
             if (not path) or not path[0]:
                 return False
-            record = model.group[path[0][0]]
+            group = model.group
+            record = group[path[0][0]]
 
-            if hasattr(path[1], '_type') and path[1]._type == 'many2one':
-                value = record[path[1].name].get(record)
-                args = ('model', 'ir.action.keyword', 'get_keyword',
-                        'form_relate', (self.screen.group.fields[
-                            path[1].name].attrs['relation'], 0), rpc.CONTEXT)
-                try:
-                    relates = rpc.execute(*args)
-                except Exception, exception:
-                    relates = common.process_exception(exception, self.window,
-                            *args)
-                    if not relates:
-                        return False
-                menu_entries = []
-                menu_entries.append((None, None, None))
-                menu_entries.append((_('Actions'),
-                    lambda x: self.click_and_action(
-                        'form_action', value, path), 0))
-                menu_entries.append((_('Reports'),
-                    lambda x: self.click_and_action(
-                        'form_print', value, path), 0))
-                menu_entries.append((None, None, None))
-                for relate in relates:
-                    relate['string'] = relate['name']
-                    fct = lambda action: lambda x: \
-                            self.click_and_relate(action, value, path)
-                    menu_entries.append(
-                            ('... ' + relate['name'], fct(relate), 0))
-                menu = gtk.Menu()
-                for stock_id, callback, sensitivity in menu_entries:
-                    if stock_id:
-                        item = gtk.ImageMenuItem(stock_id)
-                        if callback:
-                            item.connect('activate', callback)
-                        item.set_sensitive(bool(sensitivity or value))
-                    else:
-                        item = gtk.SeparatorMenuItem()
-                    item.show()
-                    menu.append(item)
-                menu.popup(None, None, None, event.button, event.time)
+            menu = gtk.Menu()
+            copy_item = gtk.ImageMenuItem('gtk-copy')
+            copy_item.connect('activate', lambda x: self.on_copy())
+            menu.append(copy_item)
+            # Don't activate actions if parent is modified
+            parent = record.parent if record else None
+            while parent:
+                if parent.modified:
+                    record = None
+                    break
+                parent = parent.parent
+            populate(menu, group.model_name, record)
+            for col in self.widget_tree.get_columns():
+                if not col.get_visible() or not col.name:
+                    continue
+                field = group.fields[col.name]
+                model = None
+                if field.attrs['type'] == 'many2one':
+                    model = field.attrs['relation']
+                    record_id = field.get(record)
+                elif field.attrs['type'] == 'reference':
+                    value = field.get(record)
+                    if value:
+                        model, record_id = value.split(',')
+                        record_id = int(record_id)
+                if not model:
+                    continue
+                label = field.attrs['string']
+                populate(menu, model, record_id, title=label)
+            menu.show_all()
+            menu.popup(None, None, None, event.button, event.time)
+        elif event.button == 2:
+            event.button = 1
+            event.state |= gtk.gdk.MOD1_MASK
+            treeview.emit('button-press-event', event)
+            return True
         return False
 
     def click_and_relate(self, action, value, path):
@@ -660,12 +593,11 @@ class ViewList(ParserView):
         context = {}
         act = action.copy()
         if not(value):
-            message(_('You must select a record to use the relation!'),
-                    self.window)
+            message(_('You must select a record to use the relation!'))
             return False
         from tryton.gui.window.view_form.screen import Screen
         screen = Screen(self.screen.group.fields[
-            path[1].name].attrs['relation'], self.window)
+            path[1].name].attrs['relation'])
         screen.load([value])
         encoder = PYSONEncoder()
         act['domain'] = encoder.encode(screen.current_record.expr_eval(
@@ -675,10 +607,10 @@ class ViewList(ParserView):
         data['model'] = self.screen.model_name
         data['id'] = value
         data['ids'] = [value]
-        return Action._exec_action(act, self.window, data, context)
+        return Action._exec_action(act, data, context)
 
     def click_and_action(self, atype, value, path):
-        return Action.exec_keyword(atype, self.window, {
+        return Action.exec_keyword(atype, {
             'model': self.screen.group.fields[
                 path[1].name].attrs['relation'],
             'id': value or False,
@@ -691,9 +623,6 @@ class ViewList(ParserView):
                 self.store.added(group, signal[1])
             elif signal[0] == 'record-removed':
                 self.store.removed(group, signal[1])
-            elif signal[0] == 'group-cleared':
-                self.store = None
-                self.widget_tree.set_model(self.store)
         self.display()
 
     def cancel(self):
@@ -705,28 +634,33 @@ class ViewList(ParserView):
     def __getitem__(self, name):
         return None
 
-    def destroy(self):
-        if CONFIG['client.save_width_height']:
-            fields = {}
-            last_col = None
-            for col in self.widget_tree.get_columns():
-                if col.get_visible():
-                    last_col = col
-                if not hasattr(col, 'name') or not hasattr(col, 'width'):
-                    continue
-                if col.get_width() != col.width and col.get_visible():
-                    fields[col.name] = col.get_width()
-            #Don't set width for last visible columns
-            #as it depends of the screen size
-            if last_col and last_col.name in fields:
-                del fields[last_col.name]
+    def save_width_height(self):
+        if not CONFIG['client.save_width_height']:
+            return
+        fields = {}
+        last_col = None
+        for col in self.widget_tree.get_columns():
+            if col.get_visible():
+                last_col = col
+            if not hasattr(col, 'name') or not hasattr(col, 'width'):
+                continue
+            if (col.get_width() != col.width and col.get_visible()
+                    and not col.get_expand()):
+                fields[col.name] = col.get_width()
+        #Don't set width for last visible columns
+        #as it depends of the screen size
+        if last_col and last_col.name in fields:
+            del fields[last_col.name]
 
-            if fields:
-                try:
-                    rpc.execute('model', 'ir.ui.view_tree_width', 'set_width',
-                            self.screen.model_name, fields, rpc.CONTEXT)
-                except Exception:
-                    pass
+        if fields and any(fields.itervalues()):
+            try:
+                RPCExecute('model', 'ir.ui.view_tree_width', 'set_width',
+                    self.screen.model_name, fields)
+            except RPCException:
+                pass
+
+    def destroy(self):
+        self.save_width_height()
         self.widget_tree.destroy()
         self.screen = None
         self.widget_tree = None
@@ -735,14 +669,22 @@ class ViewList(ParserView):
     def __sig_switch(self, treeview, path, column):
         if column._type == 'button':
             return
-        if not self.screen.row_activate():
-            if treeview.row_expanded(path):
-                treeview.collapse_row(path)
-            else:
-                treeview.expand_row(path, False)
+        allow_similar = False
+        event = gtk.get_current_event()
+        if (event.state & gtk.gdk.MOD1_MASK
+                or event.state & gtk.gdk.SHIFT_MASK):
+            allow_similar = True
+        with Window(allow_similar=allow_similar):
+            if not self.screen.row_activate() and self.children_field:
+                if treeview.row_expanded(path):
+                    treeview.collapse_row(path)
+                else:
+                    treeview.expand_row(path, False)
 
     def __select_changed(self, tree_sel):
         previous_record = self.screen.current_record
+        if previous_record and previous_record not in previous_record.group:
+            previous_record = None
 
         if tree_sel.get_mode() == gtk.SELECTION_SINGLE:
             model, iter_ = tree_sel.get_selected()
@@ -773,7 +715,6 @@ class ViewList(ParserView):
                 return True
         self.update_children()
 
-
     def set_value(self):
         if hasattr(self.widget_tree, 'editable') \
                 and self.widget_tree.editable:
@@ -815,33 +756,28 @@ class ViewList(ParserView):
     def update_children(self):
         ids = self.sel_ids_get()
         for child in self.children:
-            value = 0.0
-            value_selected = 0.0
+            value = 0
+            value_selected = 0
             loaded = True
+            child_fieldname = self.children[child][0]
             for record in self.screen.group:
-                if not record.loaded:
+                if not record.get_loaded([child_fieldname]):
                     loaded = False
                     break
-                if record.id in ids or not ids:
-                    if not value_selected:
-                        value_selected = record.fields_get()[self.children[child][0]]\
-                                .get(record, check_load=False)
-                    else:
-                        value_selected += record.fields_get()[self.children[child][0]]\
-                                .get(record, check_load=False)
-                if not value:
-                    value = record.fields_get()[self.children[child][0]]\
-                            .get(record, check_load=False)
-                else:
-                    value += record.fields_get()[self.children[child][0]]\
-                            .get(record, check_load=False)
+                field_value = record.fields_get()[child_fieldname].get(record,
+                    check_load=False)
+                if field_value is not None:
+                    value += field_value
+                    if record.id in ids or not ids:
+                        value_selected += field_value
 
             if loaded:
-                label_str = locale.format('%.' + str(self.children[child][3]) + 'f',
-                        value_selected, True)
+                label_str = locale.format(
+                    '%.' + str(self.children[child][3]) + 'f', value_selected,
+                    True)
                 label_str += ' / '
-                label_str += locale.format('%.' + str(self.children[child][3]) + 'f',
-                        value, True)
+                label_str += locale.format(
+                    '%.' + str(self.children[child][3]) + 'f', value, True)
             else:
                 label_str = '-'
             self.children[child][2].set_text(label_str)
@@ -862,7 +798,7 @@ class ViewList(ParserView):
                     editable = renderer.get_property('activatable')
                 elif isinstance(renderer,
                         (gtk.CellRendererProgress, CellRendererButton,
-                            gtk.CellRendererPixbuf)):
+                            gtk.CellRendererPixbuf, CellRendererBinary)):
                     editable = False
                 else:
                     editable = renderer.get_property('editable')
@@ -871,17 +807,19 @@ class ViewList(ParserView):
                     break
             if path[:-1]:
                 self.widget_tree.expand_to_path(path[:-1])
-            self.widget_tree.scroll_to_cell(path, focus_column, use_align=False)
+            self.widget_tree.scroll_to_cell(path, focus_column,
+                use_align=False)
             self.widget_tree.set_cursor(path, focus_column, new)
 
     def sel_ids_get(self):
         def _func_sel_get(store, path, iter, ids):
             record = store.on_get_iter(path)
-            if record and record.id > 0:
+            if record and record.id >= 0:
                 ids.append(record.id)
         ids = []
         sel = self.widget_tree.get_selection()
-        sel.selected_foreach(_func_sel_get, ids)
+        if sel:
+            sel.selected_foreach(_func_sel_get, ids)
         return ids
 
     def selected_records(self):
@@ -904,3 +842,29 @@ class ViewList(ParserView):
                     pass
                 else:
                     renderer.set_property('editable', False)
+
+    def get_expanded_paths(self, starting_path=None, starting_id_path=None):
+        # Use id instead of position
+        # because the position may change between load
+        if not starting_path:
+            starting_path = []
+        if not starting_id_path:
+            starting_id_path = []
+        id_paths = []
+        record = self.store.on_get_iter(starting_path)
+        for path_idx in range(self.store.on_iter_n_children(record)):
+            path = starting_path + [path_idx]
+            expanded = self.widget_tree.row_expanded(tuple(path))
+            if expanded:
+                expanded_record = self.store.on_get_iter(path)
+                id_path = starting_id_path + [expanded_record.id]
+                id_paths.append(id_path)
+                child_id_paths = self.get_expanded_paths(path, id_path)
+                id_paths += child_id_paths
+        return id_paths
+
+    def expand_nodes(self, nodes):
+        for node in nodes:
+            expand_path = path_convert_id2pos(self.store, node)
+            if expand_path:
+                self.widget_tree.expand_to_path(expand_path)
