@@ -3,6 +3,8 @@
 import gtk
 import parser
 import gettext
+import gobject
+
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrenderertoggle import CellRendererToggle
 
@@ -38,7 +40,6 @@ class EditableTreeView(gtk.TreeView):
 
     def on_open_remote(self, current_record, fieldname, create, value,
             entry=None, callback=None):
-        field = current_record[fieldname]
         cell = self.cells[fieldname]
         if value != cell.get_textual_value(current_record) or not value:
             changed = True
@@ -104,7 +105,7 @@ class EditableTreeView(gtk.TreeView):
         self.grab_focus()
         if focus_column and (focus_column._type in ('boolean')):
             start_editing = False
-        return super(EditableTreeView, self).set_cursor(path, focus_column,
+        super(EditableTreeView, self).set_cursor(path, focus_column,
                 start_editing)
 
     def set_value(self):
@@ -150,11 +151,12 @@ class EditableTreeView(gtk.TreeView):
                 txt = entry.get_text()
             else:
                 txt = entry.get_active_text()
-            entry.disconnect(entry.editing_done_id)
+            entry.handler_block(entry.editing_done_id)
+
             def callback():
-                entry.editing_done_id = entry.connect('editing_done',
-                        self.on_editing_done)
-                self.set_cursor(path, column, True)
+                entry.handler_unblock(entry.editing_done_id)
+                # Must wait the edited entry came back in valid state
+                gobject.idle_add(self.set_cursor, path, column, True)
             self.on_quit_cell(record, column.name, txt, callback=callback)
         if event.keyval in self.leaving_record_events:
             fields = self.cells.keys()
@@ -179,15 +181,13 @@ class EditableTreeView(gtk.TreeView):
             new_col = self.__prev_column(column)
             self.set_cursor(path, new_col, True)
         elif event.keyval == gtk.keysyms.Up:
-            entry.disconnect(entry.editing_done_id)
+            entry.handler_block(entry.editing_done_id)
             self._key_up(path, model, column)
-            entry.editing_done_id = entry.connect('editing_done',
-                    self.on_editing_done)
+            entry.handler_unblock(entry.editing_done_id)
         elif event.keyval == gtk.keysyms.Down:
-            entry.disconnect(entry.editing_done_id)
+            entry.handler_block(entry.editing_done_id)
             self._key_down(path, model, column)
-            entry.editing_done_id = entry.connect('editing_done',
-                    self.on_editing_done)
+            entry.handler_unblock(entry.editing_done_id)
         elif event.keyval in (gtk.keysyms.Return,):
             col = None
             for column in self.get_columns():
@@ -202,19 +202,19 @@ class EditableTreeView(gtk.TreeView):
                 if column.get_visible() and editable:
                     col = column
                     break
-            entry.disconnect(entry.editing_done_id)
+            entry.handler_block(entry.editing_done_id)
             if self.editable == 'top':
-                new_path = self._key_up(path, model, col)
+                self._key_up(path, model, col)
             else:
-                new_path = self._key_down(path, model, column)
-            entry.editing_done_id = entry.connect('editing_done',
-                    self.on_editing_done)
+                self._key_down(path, model, column)
+            entry.handler_unblock(entry.editing_done_id)
         elif event.keyval in (gtk.keysyms.F3, gtk.keysyms.F2):
             if isinstance(entry, gtk.Entry):
                 value = entry.get_text()
             else:
                 value = entry.get_active_text()
-            entry.disconnect(entry.editing_done_id)
+            entry.handler_block(entry.editing_done_id)
+
             def callback():
                 cell = self.cells[column.name]
                 value = cell.get_textual_value(record)
@@ -222,17 +222,16 @@ class EditableTreeView(gtk.TreeView):
                     entry.set_text(value)
                 else:
                     entry.set_active_text(value)
-                entry.editing_done_id = entry.connect('editing_done',
-                        self.on_editing_done)
-                self.set_cursor(path, column, True)
+                entry.handler_unblock(entry.editing_done_id)
             self.on_open_remote(record, column.name,
-                create=(event.keyval==gtk.keysyms.F3), value=value,
+                create=(event.keyval == gtk.keysyms.F3), value=value,
                 callback=callback)
         else:
             field = record[column.name]
             if isinstance(entry, gtk.Entry):
                 entry.set_max_length(int(field.attrs.get('size', 0)))
-            # store in the record the entry widget to get the value in set_value
+            # store in the record the entry widget to get the value in
+            # set_value
             field.editabletree_entry = entry
             record.modified_fields.setdefault(column.name)
             return False
