@@ -1,18 +1,25 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 
+import os
 import gobject
+import tempfile
+import gtk
+import locale
+import gettext
+import operator
+
+from functools import wraps
+
 from editabletree import EditableTreeView
 from tryton.gui.window.view_form.view.interface import ParserInterface
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
 from tryton.gui.window.view_form.screen import Screen
-from tryton.config import CONFIG
 import tryton.rpc as rpc
-from tryton.common import DT_FORMAT, DHM_FORMAT, COLORS, node_attributes, \
-        TRYTON_ICON, HM_FORMAT
+from tryton.common import COLORS, node_attributes, \
+        HM_FORMAT, file_selection, file_open
 import tryton.common as common
-from tryton.exceptions import TrytonError, TrytonServerError
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrendererdate import CellRendererDate
 from tryton.common.cellrenderertext import CellRendererText
@@ -20,18 +27,14 @@ from tryton.common.cellrenderertoggle import CellRendererToggle
 from tryton.common.cellrenderercombo import CellRendererCombo
 from tryton.common.cellrendererinteger import CellRendererInteger
 from tryton.common.cellrendererfloat import CellRendererFloat
+from tryton.common.cellrendererbinary import CellRendererBinary
 from tryton.action import Action
 from tryton.translate import date_format
-from tryton.pyson import PYSONDecoder
-import gtk
-import locale
-import datetime
-import time
-import gettext
-import operator
+from tryton.common import RPCExecute, RPCException
 
 _ = gettext.gettext
 
+
 def send_keys(renderer, editable, position, treeview):
     editable.connect('key_press_event', treeview.on_keypressed)
     editable.editing_done_id = editable.connect('editing_done',
@@ -39,6 +42,7 @@ def send_keys(renderer, editable, position, treeview):
     if isinstance(editable, (gtk.ComboBoxEntry, gtk.ComboBox)):
         editable.connect('changed', treeview.on_editing_done)
 
+
 def sort_model(column, treeview, screen):
     for col in treeview.get_columns():
         if col != column:
@@ -68,6 +72,18 @@ def sort_model(column, treeview, screen):
     for record in unsaved_records:
         store.group.append(record)
 
+
+def realized(func):
+    "Decorator for treeview realized"
+    @wraps(func)
+    def wrapper(self, *args, **kwargs):
+        if (hasattr(self.treeview, 'get_realized')
+                and not self.treeview.get_realized()):
+            return
+        return func(self, *args, **kwargs)
+    return wrapper
+
+
 class ParserTree(ParserInterface):
 
     def __init__(self, parent=None, attrs=None, screen=None,
@@ -90,17 +106,19 @@ class ParserTree(ParserInterface):
         treeview.sequence = attrs.get('sequence', False)
         treeview.colors = attrs.get('colors', '"black"')
         treeview.keyword_open = attrs.get('keyword_open', False)
+        treeview.connect('focus', self.set_selection)
         self.treeview = treeview
         treeview.set_property('rules-hint', True)
         if not self.title:
             self.title = attrs.get('string', 'Unknown')
         tooltips = common.Tooltips()
+        expandable = False
 
         for node in root_node.childNodes:
             node_attrs = node_attributes(node)
             if node.localName == 'field':
                 fname = str(node_attrs['name'])
-                for boolean_fields in ('readonly', 'required'):
+                for boolean_fields in ('readonly', 'required', 'expand'):
                     if boolean_fields in node_attrs:
                         node_attrs[boolean_fields] = \
                                 bool(int(node_attrs[boolean_fields]))
@@ -108,7 +126,7 @@ class ParserTree(ParserInterface):
                     continue
                 for attr_name in ('relation', 'domain', 'selection',
                         'relation_field', 'string', 'views', 'invisible',
-                        'add_remove', 'sort', 'context'):
+                        'add_remove', 'sort', 'context', 'filename'):
                     if attr_name in fields[fname].attrs and \
                             not attr_name in node_attrs:
                         node_attrs[attr_name] = fields[fname].attrs[attr_name]
@@ -127,7 +145,8 @@ class ParserTree(ParserInterface):
                     pass
                 else:
                     renderer.set_property('editable', not readonly)
-                if not readonly:
+                if (not readonly
+                        and not isinstance(renderer, CellRendererBinary)):
                     renderer.connect_after('editing-started', send_keys,
                             treeview)
 
@@ -137,7 +156,11 @@ class ParserTree(ParserInterface):
                     render_pixbuf = gtk.CellRendererPixbuf()
                     col.pack_start(render_pixbuf, expand=False)
                     icon = node_attrs['icon']
+
                     def setter(column, cell, store, iter):
+                        if (hasattr(self.treeview, 'get_realized')
+                                and not self.treeview.get_realized()):
+                            return
                         record = store.get_value(iter, 0)
                         value = record[icon].get_client(record) or ''
                         common.ICONFACTORY.register_icon(value)
@@ -181,6 +204,7 @@ class ParserTree(ParserInterface):
                     'one2many': 50,
                     'many2many': 50,
                     'boolean': 20,
+                    'binary': 200,
                 }
                 if 'width' in node_attrs:
                     width = int(node_attrs['width'])
@@ -190,8 +214,9 @@ class ParserTree(ParserInterface):
                 if width > 0:
                     col.set_fixed_width(width)
                 col.set_min_width(1)
-                #XXX doesn't work well when resize columns
-                #col.set_expand(True)
+                expand = node_attrs.get('expand', False)
+                col.set_expand(expand)
+                expandable |= expand
                 if (not treeview.sequence
                         and not self.children_field
                         and fields[fname].attrs.get('sortable', True)):
@@ -236,6 +261,7 @@ class ParserTree(ParserInterface):
                 col.set_widget(label)
 
                 col._type = 'button'
+                col.set_cell_data_func(renderer, cell.setter)
                 if 'width' in node_attrs:
                     width = int(node_attrs['width'])
                 else:
@@ -246,7 +272,7 @@ class ParserTree(ParserInterface):
                 col.set_resizable(True)
                 col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
                 i = treeview.append_column(col)
-        if not bool(int(attrs.get('fill', '0'))):
+        if not expandable:
             col = gtk.TreeViewColumn()
             col.name = None
             arrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_IN)
@@ -258,6 +284,12 @@ class ParserTree(ParserInterface):
         treeview.set_fixed_height_mode(True)
         return treeview, dict_widget, button_list, on_write, [], None
 
+    def set_selection(self, treeview, direction):
+        selection = treeview.get_selection()
+        if len(treeview.get_model()):
+            selection.select_path(0)
+        return False
+
 
 class Char(object):
 
@@ -267,8 +299,10 @@ class Char(object):
         self.model_name = model_name
         self.attrs = attrs or {}
         self.renderer = CellRendererText()
+        self.renderer.connect('editing-started', self.editing_started)
         self.treeview = treeview
 
+    @realized
     def setter(self, column, cell, store, iter):
         record = store.get_value(iter, 0)
         text = self.get_textual_value(record)
@@ -316,7 +350,8 @@ class Char(object):
                 bg_color = 'white'
                 if not field.get_state_attrs(record).get('valid', True):
                     bg_color = COLORS.get('invalid', 'white')
-                elif bool(int(field.get_state_attrs(record).get('required', 0))):
+                elif bool(int(
+                            field.get_state_attrs(record).get('required', 0))):
                     bg_color = COLORS.get('required', 'white')
                 cell.set_property('background', bg_color)
                 if bg_color == 'white':
@@ -346,7 +381,7 @@ class Char(object):
     def get_textual_value(self, record):
         if not record:
             return ''
-        return record[self.field_name].get_client(record) or ''
+        return record[self.field_name].get_client(record)
 
     def value_from_text(self, record, text, callback=None):
         field = record[self.field_name]
@@ -354,22 +389,18 @@ class Char(object):
         if callback:
             callback()
 
+    def editing_started(self, cell, editable, path):
+        return False
+
+
 class Int(Char):
 
     def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Int, self).__init__(field_name, model_name, treeview,
             attrs=attrs)
         self.renderer = CellRendererInteger()
+        self.renderer.connect('editing-started', self.editing_started)
 
-    def value_from_text(self, record, text, callback=None):
-        field = record[self.field_name]
-        field.set_client(record, int(text))
-        if callback:
-            callback()
-
-    def get_textual_value(self, record):
-        return locale.format('%d',
-                record[self.field_name].get_client(record) or 0, True)
 
 class Boolean(Int):
 
@@ -379,9 +410,6 @@ class Boolean(Int):
         self.renderer = CellRendererToggle()
         self.renderer.connect('toggled', self._sig_toggled)
 
-    def get_textual_value(self, record):
-        return record[self.field_name].get_client(record)
-
     def _sig_toggled(self, renderer, path):
         store = self.treeview.get_model()
         record = store.get_value(store.get_iter(path), 0)
@@ -394,65 +422,28 @@ class Boolean(Int):
 
 
 class Date(Char):
-    server_format = DT_FORMAT
 
     def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Date, self).__init__(field_name, model_name, treeview,
             attrs=attrs)
-        self.display_format = date_format()
-        self.renderer = CellRendererDate(self.display_format)
-
-    def get_textual_value(self, record):
-        value = record[self.field_name].get_client(record)
-        if not value:
-            return ''
-        date = datetime.date(*time.strptime(value, self.server_format)[:3])
-        return common.datetime_strftime(date, self.display_format)
-
-    def value_from_text(self, record, text, callback=None):
-        field = record[self.field_name]
-        date = False
-        try:
-            date = datetime.date(*time.strptime(text, self.display_format)[:3])
-            date = common.datetime_strftime(date, self.server_format)
-        except ValueError:
-            date = False
-        field.set_client(record, date)
-        if callback:
-            callback()
+        self.renderer = CellRendererDate(date_format())
+        self.renderer.connect('editing-started', self.editing_started)
 
 
 class Datetime(Date):
-    server_format = DHM_FORMAT
 
     def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Datetime, self).__init__(field_name, model_name, treeview,
             attrs=attrs)
-        self.display_format = date_format() + ' ' + HM_FORMAT
-        self.renderer.format = self.display_format
+        self.renderer.format = date_format() + ' ' + HM_FORMAT
 
-    def get_textual_value(self, record):
-        value = record[self.field_name].get_client(record)
-        if not value:
-            return ''
-        date = datetime.datetime(*time.strptime(value,
-            self.server_format)[:6])
-        date = common.timezoned_date(date)
-        return common.datetime_strftime(date, self.display_format)
 
-    def value_from_text(self, record, text, callback=None):
-        field = record[self.field_name]
-        date = False
-        try:
-            date = datetime.datetime(*time.strptime(text,
-                self.display_format)[:6])
-            date = common.timezoned_date(date)
-            date = common.datetime_strftime(date, self.server_format)
-        except ValueError:
-            date = False
-        field.set_client(record, date)
-        if callback:
-            callback()
+class Time(Date):
+
+    def __init__(self, field_name, model_name, treeview, attrs=None):
+        super(Time, self).__init__(field_name, model_name, treeview,
+            attrs=attrs)
+        self.renderer.format = HM_FORMAT
 
 
 class Float(Char):
@@ -461,30 +452,16 @@ class Float(Char):
         super(Float, self).__init__(field_name, model_name, treeview,
             attrs=attrs)
         self.renderer = CellRendererFloat()
+        self.renderer.connect('editing-started', self.editing_started)
 
+    @realized
     def setter(self, column, cell, store, iter):
         super(Float, self).setter(column, cell, store, iter)
         record = store.get_value(iter, 0)
         field = record[self.field_name]
-        digits = record.expr_eval(field.attrs.get('digits', (16, 2)))
+        digits = field.digits(record)
         cell.digits = digits
 
-    def get_textual_value(self, record):
-        field = record[self.field_name]
-        digit = record.expr_eval(field.attrs.get('digits', (16, 2)))[1]
-        return locale.format('%.'+str(digit)+'f',
-                record[self.field_name].get_client(record) or 0.0, True)
-
-    def value_from_text(self, record, text, callback=None):
-        field = record[self.field_name]
-        try:
-            value = locale.atof(text)
-        except ValueError:
-            value = 0.0
-        field.set_client(record, value)
-        if callback:
-            callback()
-
 
 class FloatTime(Char):
 
@@ -496,44 +473,115 @@ class FloatTime(Char):
             self.conv = rpc.CONTEXT.get(attrs['float_time'])
 
     def get_textual_value(self, record):
-        val = record[self.field_name].get_client(record)
+        val = record[self.field_name].get(record)
         return common.float_time_to_text(val, self.conv)
 
     def value_from_text(self, record, text, callback=None):
         field = record[self.field_name]
-        field.set_client(record, common.text_to_float_time(text, self.conv))
+        digits = field.digits(record)
+        field.set_client(record,
+            common.text_to_float_time(text, self.conv, digits[1]))
         if callback:
             callback()
 
+
+class Binary(Char):
+
+    def __init__(self, field_name, model_name, treeview, attrs=None):
+        super(Binary, self).__init__(field_name, model_name, treeview,
+            attrs=attrs)
+        self.filename = attrs.get('filename')
+        self.renderer = CellRendererBinary(bool(self.filename))
+        self.renderer.connect('new', self.new_binary)
+        self.renderer.connect('open', self.open_binary)
+        self.renderer.connect('save', self.save_binary)
+        self.renderer.connect('clear', self.clear_binary)
+
+    def get_textual_value(self, record):
+        pass
+
+    def value_from_text(self, record, text, callback=None):
+        if callback:
+            callback()
+
+    @realized
+    def setter(self, column, cell, store, iter):
+        record = store.get_value(iter, 0)
+        size = record[self.field_name].get_size(record)
+        cell.set_property('size', common.humanize(size) if size else '')
+
+    def _get_record_field(self, path):
+        store = self.treeview.get_model()
+        record = store.get_value(store.get_iter(path), 0)
+        field = record.group.fields[self.field_name]
+        return record, field
+
+    def new_binary(self, renderer, path):
+        filename = file_selection(_('Open...'))
+        record, field = self._get_record_field(path)
+        if filename:
+            field.set_client(record, open(filename, 'rb').read())
+            if self.filename:
+                filename_field = record.group.fields[self.filename]
+                filename_field.set_client(record, os.path.basename(filename))
+
+    def open_binary(self, renderer, path):
+        if not self.filename:
+            return
+        dtemp = tempfile.mkdtemp(prefix='tryton_')
+        record, field = self._get_record_field(path)
+        filename_field = record.group.fields.get(self.filename)
+        filename = filename_field.get(record).replace(
+            os.sep, '_').replace(os.altsep or os.sep, '_')
+        if not filename:
+            return
+        file_path = os.path.join(dtemp, filename)
+        with open(file_path, 'wb') as fp:
+            fp.write(field.get_data(record))
+        root, type_ = os.path.splitext(filename)
+        if type_:
+            type_ = type_[1:]
+        file_open(file_path, type_)
+
+    def save_binary(self, renderer, path):
+        filename = ''
+        record, field = self._get_record_field(path)
+        if self.filename:
+            filename_field = record.group.fields.get(self.filename)
+            filename = filename_field.get(record)
+        filename = file_selection(_('Save As...'), filename=filename,
+            action=gtk.FILE_CHOOSER_ACTION_SAVE)
+        if filename:
+            with open(filename, 'wb') as fp:
+                fp.write(field.get_data(record))
+
+    def clear_binary(self, renderer, path):
+        record, field = self._get_record_field(path)
+        field.set_client(record, False)
+
+
 class M2O(Char):
 
     def value_from_text(self, record, text, callback=None):
         field = record.group.fields[self.field_name]
-        if not text and not field.get_state_attrs(
-                record)['required']:
+        if not text:
+            field.set_client(record, (None, ''))
             if callback:
                 callback()
-            return False
+            return
 
         relation = record[self.field_name].attrs['relation']
         domain = record[self.field_name].domain_get(record)
         context = record[self.field_name].context_get(record)
-        if text:
-            dom = [('rec_name', 'ilike', '%' + text + '%'),
-                    domain]
-        else:
-            dom = domain
-        args = ('model', relation, 'search', dom, 0, None, None,
-                context)
+        dom = [('rec_name', 'ilike', '%' + text + '%'), domain]
         try:
-            ids = rpc.execute(*args)
-        except TrytonServerError, exception:
-            ids = common.process_exception(exception, *args)
-            if not ids:
-                field.set_client(record, '???')
-                if callback:
-                    callback()
-                return
+            ids = RPCExecute('model', relation, 'search', dom, 0, None, None,
+                context=context)
+        except RPCException:
+            field.set_client(record, (None, ''))
+            if callback:
+                callback()
+            return
         if len(ids) != 1:
             if callback:
                 self.search_remote(record, relation, ids, domain=domain,
@@ -559,16 +607,14 @@ class M2O(Char):
                 dom = [('rec_name', 'ilike', '%' + text + '%'), domain]
             else:
                 dom = domain
-            args = ('model', relation, 'search', dom, 0, None, None, context)
             try:
-                ids = rpc.execute(*args)
-            except TrytonServerError, exception:
-                ids = common.process_exception(exception, *args)
-                if not ids:
-                    field.set_client(record, False)
-                    if callback:
-                        callback()
-                    return
+                ids = RPCExecute('model', relation, 'search', dom, 0, None,
+                    None, context=context)
+            except RPCException:
+                field.set_client(record, False)
+                if callback:
+                    callback()
+                return
             if len(ids) == 1:
                 field.set_client(record, ids[0])
                 if callback:
@@ -584,7 +630,10 @@ class M2O(Char):
             if result and screen.save_current():
                 value = (screen.current_record.id,
                     screen.current_record.rec_name())
-                field.set_client(record, value)
+                field.set_client(record, value, force_change=True)
+            elif result:
+                screen.display()
+                return WinForm(screen, open_callback)
             if callback:
                 callback()
         if obj_id:
@@ -596,17 +645,11 @@ class M2O(Char):
     def search_remote(self, record, relation, ids=None, domain=None,
             context=None, callback=None):
         field = record.group.fields[self.field_name]
+
         def search_callback(found):
             value = None
             if found:
-                args = ('model', relation, 'read', found[0], ['rec_name'],
-                        context)
-                try:
-                    res = rpc.execute(*args)
-                except TrytonServerError, exception:
-                    res = common.process_exception(exception, *args)
-                if res:
-                    value = (found[0], res['rec_name'])
+                value = found[0]
             field.set_client(record, value)
             if callback:
                 callback()
@@ -626,13 +669,14 @@ class UnsettableColumn(Exception):
 
 class O2M(Char):
 
+    @realized
     def setter(self, column, cell, store, iter):
         super(O2M, self).setter(column, cell, store, iter)
         cell.set_property('xalign', 0.5)
 
     def get_textual_value(self, record):
         return '( ' + str(len(record[self.field_name].\
-                get_client(record))) + ' )'
+                get_eval(record))) + ' )'
 
     def value_from_text(self, record, text, callback=None):
         if callback:
@@ -648,94 +692,32 @@ class O2M(Char):
         screen = Screen(relation, mode=['tree', 'form'],
             exclude_field=field.attrs.get('relation_field'))
         screen.group = group
+
         def open_callback(result):
             if callback:
                 callback()
         WinForm(screen, open_callback, view_type='tree', context=context)
 
 
-class M2M(Char):
-
-    def setter(self, column, cell, store, iter):
-        super(M2M, self).setter(column, cell, store, iter)
-        cell.set_property('xalign', 0.5)
-
-    def get_textual_value(self, record):
-        return '( ' + str(len(record[self.field_name].\
-                get_client(record))) + ' )'
-
-    def value_from_text(self, record, text, callback=None):
-        field = record[self.field_name]
-        if not text:
-            field.set_client(record, [])
-            if callback:
-                callback()
-            return
-        if not (text[0] != '('):
-            if callback:
-                callback()
-            return
-        relation = field.attrs['relation']
-        domain = field.domain_get(record)
-        context = field.context_get(record)
-        if text:
-            dom = [('rec_name', 'ilike', '%' + text + '%'),
-                    domain]
-        else:
-            dom = domain
-        args = ('model', relation, 'search', dom, 0, CONFIG['client.limit'],
-                None, context)
-        try:
-            ids = rpc.execute(*args)
-        except TrytonServerError, exception:
-            ids = common.process_exception(exception, *args)
-            if ids is False:
-                field.set_client(record, [])
-                if callback:
-                    callback()
-                return
-        if not callback:
-            return
-        def winsearch_callback(result):
-            field.set_client(record, result or [])
-            if callback:
-                callback()
-        WinSearch(relation, winsearch_callback, sel_multi=True, ids=ids,
-            context=context, domain=domain)
-        return
+class M2M(O2M):
 
     def open_remote(self, record, create=True, changed=False, text=None,
             callback=None):
-        field = record[self.field_name]
+        group = record.value[self.field_name]
+        field = record.group.fields[self.field_name]
         relation = field.attrs['relation']
         context = field.context_get(record)
         domain = field.domain_get(record)
-        if create:
-            if text and len(text) and text[0] != '(':
-                domain.append(('name', '=', text))
-            args = ('model', relation, 'search', domain)
-            try:
-                ids = rpc.execute(*args)
-            except TrytonServerError, exception:
-                ids = common.process_exception(exception, *args)
-                if ids is False:
-                    field.set_client(record, False)
-                    if callback:
-                        callback()
-            if ids and len(ids)==1:
-                field.set_client(record, ids)
-                if callback:
-                    callback()
-                return
-        else:
-            ids = [x.id for x in field.get_client(record)]
+
+        screen = Screen(relation, mode=['tree', 'form'],
+            exclude_field=field.attrs.get('relation_field'))
+        screen.group = group
+
         def open_callback(result):
-            if result:
-                field.set_client(record, result)
             if callback:
                 callback()
-        WinSearch(relation, open_callback, sel_multi=True, ids=ids, context=context,
-            domain=domain)
+        WinForm(screen, open_callback, view_type='tree', domain=domain,
+            context=context)
 
 
 class Selection(Char):
@@ -748,11 +730,10 @@ class Selection(Char):
         self._domain_cache = {}
         selection = self.attrs.get('selection', [])[:]
         if not isinstance(selection, (list, tuple)):
-            args = ('model', self.model_name, selection, rpc.CONTEXT)
             try:
-                selection = rpc.execute(*args)
-            except TrytonServerError, exception:
-                selection = (common.process_exception(exception, args) or [])
+                selection = RPCExecute('model', self.model_name, selection)
+            except RPCException:
+                selection = []
         self.selection = selection[:]
         if self.attrs.get('sort', True):
             selection.sort(key=operator.itemgetter(1))
@@ -774,8 +755,6 @@ class Selection(Char):
     def get_textual_value(self, record):
         self.update_selection(record)
         value = record[self.field_name].get(record)
-        if isinstance(value, (list, tuple)):
-            value = value[0]
         return dict(self.selection).get(value, '')
 
     def value_from_text(self, record, text, callback=None):
@@ -785,6 +764,7 @@ class Selection(Char):
             callback()
 
     def editing_started(self, cell, editable, path):
+        super(Selection, self).editing_started(cell, editable, path)
         store = self.treeview.get_model()
         record = store.get_value(store.get_iter(path), 0)
         self.update_selection(record)
@@ -811,12 +791,11 @@ class Selection(Char):
             self.selection = self._domain_cache[str(domain)]
             self._last_domain = domain
         if domain != self._last_domain:
-            args = ('model', self.attrs['relation'], 'search_read', domain,
-                0, None, None, ['rec_name'], rpc.CONTEXT)
             try:
-                result = rpc.execute(*args)
-            except TrytonServerError, exception:
-                result = common.process_exception(exception, *args)
+                result = RPCExecute('model', self.attrs['relation'],
+                    'search_read', domain, 0, None, None, ['rec_name'])
+            except RPCException:
+                result = None
 
             if isinstance(result, list):
                 selection = [(x['id'], x['rec_name']) for x in result]
@@ -840,10 +819,8 @@ class Reference(Char):
         selection = attrs.get('selection', [])
         if not isinstance(selection, (list, tuple)):
             try:
-                selection = rpc.execute('model',
-                        model_name, selection, rpc.CONTEXT)
-            except TrytonServerError, exception:
-                common.process_exception(exception)
+                selection = RPCExecute('model', model_name, selection)
+            except RPCException:
                 selection = []
         selection.sort(key=operator.itemgetter(1))
         for i, j in selection:
@@ -852,9 +829,9 @@ class Reference(Char):
     def get_textual_value(self, record):
         value = record[self.field_name].get_client(record)
         if not value:
-            model, (obj_id, name) = '', (-1, '')
+            model, name = '', ''
         else:
-            model, (obj_id, name) = value
+            model, name = value
         if model:
             return self._selection.get(model, model) + ',' + name
         else:
@@ -884,11 +861,13 @@ class ProgressBar(object):
         self.renderer.set_property('orientation', orientation)
         self.treeview = treeview
 
+    @realized
     def setter(self, column, cell, store, iter):
         record = store.get_value(iter, 0)
+        field = record[self.field_name]
         value = float(self.get_textual_value(record) or 0.0)
         cell.set_property('value', value)
-        digit = record.expr_eval(self.attrs.get('digits', (16, 2)))[1]
+        digit = field.digits(record)[1]
         text = locale.format('%.' + str(digit) + 'f', value, True)
         cell.set_property('text', text + '%')
 
@@ -917,6 +896,23 @@ class Button(object):
 
         self.renderer.connect('clicked', self.button_clicked)
 
+    @realized
+    def setter(self, column, cell, store, iter):
+        record = store.get_value(iter, 0)
+        states = record.expr_eval(self.attrs.get('states', {}),
+            check_load=False)
+        invisible = states.get('invisible', False)
+        cell.set_property('visible', not invisible)
+        readonly = states.get('readonly', False)
+        cell.set_property('sensitive', not readonly)
+        parent = record.parent if record else None
+        while parent:
+            if parent.modified:
+                cell.set_property('sensitive', False)
+                break
+            parent = parent.parent
+        # TODO icon
+
     def button_clicked(self, widget, path):
         if not path:
             return True
@@ -934,32 +930,21 @@ class Button(object):
         if obj_id:
             if not self.attrs.get('confirm', False) or \
                     common.sur(self.attrs['confirm']):
-                button_type = self.attrs.get('type', 'workflow')
-                ctx = rpc.CONTEXT.copy()
-                ctx.update(record.context_get())
-                if button_type == 'workflow':
-                    args = ('model', self.screen.model_name,
-                            'workflow_trigger_validate', obj_id,
-                            self.attrs['name'], ctx)
+                button_type = self.attrs.get('type', 'object')
+                ctx = record.context_get()
+                if button_type == 'object':
                     try:
-                        rpc.execute(*args)
-                    except TrytonServerError, exception:
-                        common.process_exception(exception, *args)
-                elif button_type == 'object':
-                    args = ('model', self.screen.model_name,
-                            self.attrs['name'], [obj_id], ctx)
-                    try:
-                        rpc.execute(*args)
-                    except TrytonServerError, exception:
-                        common.process_exception(exception, *args)
+                        RPCExecute('model', self.screen.model_name,
+                            self.attrs['name'], [obj_id], context=ctx)
+                    except RPCException:
+                        pass
                 elif button_type == 'action':
-                    action_id = None
-                    args = ('model', 'ir.action', 'get_action_id',
-                            int(self.attrs['name']), ctx)
                     try:
-                        action_id = rpc.execute(*args)
-                    except TrytonServerError, exception:
-                        action_id = common.process_exception(exception, *args)
+                        action_id = RPCExecute('model', 'ir.action',
+                            'get_action_id', int(self.attrs['name']),
+                            context=ctx)
+                    except RPCException:
+                        action_id = None
                     if action_id:
                         Action.execute(action_id, {
                             'model': self.screen.model_name,
@@ -967,7 +952,7 @@ class Button(object):
                             'ids': [obj_id],
                             }, context=ctx)
                 else:
-                    raise TrytonError('Unallowed button type')
+                    raise Exception('Unallowed button type')
                 self.screen.reload(written=True)
             else:
                 self.screen.display()
@@ -985,6 +970,7 @@ CELLTYPES = {
     'integer': Int,
     'biginteger': Int,
     'datetime': Datetime,
+    'time': Time,
     'boolean': Boolean,
     'text': Char,
     'url': Char,
@@ -994,4 +980,5 @@ CELLTYPES = {
     'progressbar': ProgressBar,
     'reference': Reference,
     'one2one': O2O,
+    'binary': Binary,
 }
