@@ -4,10 +4,9 @@ import gtk
 import gobject
 import gettext
 import tryton.common as common
-import tryton.rpc as rpc
 import csv
 from tryton.config import TRYTON_ICON, CONFIG
-from tryton.exceptions import TrytonServerError
+from tryton.common import RPCExecute, RPCException
 
 _ = gettext.gettext
 
@@ -45,7 +44,7 @@ class WinImport(object):
         hbox_mapping.pack_start(vbox_buttons, False, True, 0)
 
         button_add = gtk.Button(_("_Add"), stock=None, use_underline=True)
-        button_add.set_alignment(0.0,0.0)
+        button_add.set_alignment(0.0, 0.0)
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-list-add', gtk.ICON_SIZE_BUTTON)
         button_add.set_image(img_button)
@@ -54,7 +53,7 @@ class WinImport(object):
 
         button_remove = gtk.Button(_("_Remove"), stock=None,
                 use_underline=True)
-        button_remove.set_alignment(0.0,0.0)
+        button_remove.set_alignment(0.0, 0.0)
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-list-remove', gtk.ICON_SIZE_BUTTON)
         button_remove.set_image(img_button)
@@ -63,7 +62,7 @@ class WinImport(object):
 
         button_remove_all = gtk.Button(_("Clear"), stock=None,
                 use_underline=True)
-        button_remove_all.set_alignment(0.0,0.0)
+        button_remove_all.set_alignment(0.0, 0.0)
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-clear', gtk.ICON_SIZE_BUTTON)
         button_remove_all.set_image(img_button)
@@ -75,7 +74,7 @@ class WinImport(object):
 
         button_autodetect = gtk.Button(_("Auto-Detect"), stock=None,
                 use_underline=True)
-        button_autodetect.set_alignment(0.0,0.0)
+        button_autodetect.set_alignment(0.0, 0.0)
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-find', gtk.ICON_SIZE_BUTTON)
         button_autodetect.set_image(img_button)
@@ -132,26 +131,23 @@ class WinImport(object):
         table.attach(self.import_csv_sep, 1, 2, 0, 1)
 
         label_import_csv_del = gtk.Label(_("Text Delimiter:"))
-        label_import_csv_del.set_alignment( 1, 0.5)
+        label_import_csv_del.set_alignment(1, 0.5)
         table.attach(label_import_csv_del, 2, 3, 0, 1)
         self.import_csv_del = gtk.Entry()
-        self.import_csv_del.set_text( "\"")
-        self.import_csv_del.set_width_chars( 1)
+        self.import_csv_del.set_text("\"")
+        self.import_csv_del.set_width_chars(1)
         table.attach(self.import_csv_del, 3, 4, 0, 1)
 
         label_import_csv_enc = gtk.Label(_("Encoding:"))
-        label_import_csv_enc.set_alignment( 1, 0.5)
+        label_import_csv_enc.set_alignment(1, 0.5)
         table.attach(label_import_csv_enc, 0, 1, 1, 2)
         self.import_csv_enc = gtk.combo_box_new_text()
         self.import_csv_enc.append_text("UTF-8")
         self.import_csv_enc.append_text("Latin1")
-        cboxent_import_csv_enc = gtk.Entry()
-        cboxent_import_csv_enc.unset_flags(gtk.CAN_FOCUS)
-        self.import_csv_enc.add(cboxent_import_csv_enc)
         table.attach(self.import_csv_enc, 1, 2, 1, 2)
 
         label_import_csv_skip = gtk.Label(_("Lines to Skip:"))
-        label_import_csv_skip.set_alignment( 1, 0.5)
+        label_import_csv_skip.set_alignment(1, 0.5)
         table.attach(label_import_csv_skip, 2, 3, 1, 2)
 
         self.import_csv_skip_adj = gtk.Adjustment(0, 0, 100, 1, 10)
@@ -227,11 +223,10 @@ class WinImport(object):
                     self.model1.insert(node, 0, [None, '', 'white'])
 
     def _get_fields(self, model):
-        args = ('model', model, 'fields_get', None, rpc.CONTEXT)
         try:
-            return rpc.execute(*args)
-        except TrytonServerError, exception:
-            return common.process_exception(exception, self.dialog, *args)
+            return RPCExecute('model', model, 'fields_get', None)
+        except RPCException:
+            return ''
 
     def on_row_expanded(self, treeview, iter, path):
         child = self.model1.iter_children(iter)
@@ -245,8 +240,7 @@ class WinImport(object):
     def sig_autodetect(self, widget=None):
         fname = self.import_csv_file.get_filename()
         if not fname:
-            common.message(_('You must select an import file first!'),
-                    self.dialog)
+            common.message(_('You must select an import file first!'))
             return True
         csvsep = self.import_csv_sep.get_text()
         csvdel = self.import_csv_del.get_text()
@@ -258,8 +252,7 @@ class WinImport(object):
             data = csv.reader(open(fname, 'rb'), quotechar=csvdel,
                     delimiter=csvsep)
         except IOError:
-            common.warning(_('Error opening CSV file'), self.dialog,
-                    _('Error'))
+            common.warning(_('Error opening CSV file'), _('Error'))
             return True
         self.sig_unsel_all()
         word = ''
@@ -269,7 +262,6 @@ class WinImport(object):
                 if word not in self.fields_invert and word not in self.fields:
                     iter = self.model1.get_iter_first()
                     prefix = ''
-                    prefix_name = ''
                     for parent in word.split('/')[:-1]:
                         while iter:
                             if self.model1.get_value(iter, 0) == parent or \
@@ -287,9 +279,8 @@ class WinImport(object):
                     name = self.fields[word][0]
                     field = word
                 else:
-                    common.warning(
-                            _('Error processing the file at field %s.') %
-                            word, self.dialog, _('Error'))
+                    common.warning(_('Error processing the file at field %s.')
+                        % word, _('Error'))
                     return True
                 num = self.model2.append()
                 self.model2.set(num, 0, name, 1, field)
@@ -303,7 +294,7 @@ class WinImport(object):
     def _sig_sel_add(self, store, path, iter):
         num = self.model2.append()
         name = self.fields[store.get_value(iter, 1)][0]
-        self.model2.set(num, 0, name, 1, store.get_value(iter,1))
+        self.model2.set(num, 0, name, 1, store.get_value(iter, 1))
 
     def sig_unsel(self, widget=None):
         store, paths = self.view2.get_selection().get_selected_rows()
@@ -353,10 +344,8 @@ class WinImport(object):
             datas.append([x.decode(csv_data['combo']).encode('utf-8') \
                     for x in line])
         try:
-            res = rpc.execute('model', model, 'import_data', fields, datas,
-                    rpc.CONTEXT)
-        except TrytonServerError, exception:
-            common.process_exception(exception, self.dialog)
+            res = RPCExecute('model', model, 'import_data', fields, datas)
+        except RPCException:
             return False
         if res[0] >= 0:
             if res[0] == 1:
@@ -368,6 +357,7 @@ class WinImport(object):
             for key, val in res[1].items():
                 buf += ('\t%s: %s\n' % (str(key), str(val)))
             common.error(_('Importation Error!'),
-                    _('Error trying to import this record:\n' \
-                    '%s\nError Message:\n%s\n\n%s') % (buf, res[2], res[3]))
+                _('Error importing record %(record)s\n'
+                    '%(error_title)s\n\n%(traceback)s') %
+                {'record': buf, 'error_title': res[2], 'traceback': res[3]})
         return True
